@@ -53,7 +53,6 @@ bytes [#bytes]_                          binary         both
 .. [#bytes] The bytes type is encoded as BSON binary with
    subtype 0. It will be decoded back to bytes.
 """
-
 import datetime
 import itertools
 import os
@@ -84,6 +83,7 @@ from typing import (
     TypeVar,
     Union,
     cast,
+    overload,
 )
 
 from bson.binary import (
@@ -101,7 +101,6 @@ from bson.codec_options import (
     DEFAULT_CODEC_OPTIONS,
     CodecOptions,
     DatetimeConversion,
-    _DocumentType,
     _raw_document_class,
 )
 from bson.datetime_ms import (
@@ -125,8 +124,7 @@ from bson.timestamp import Timestamp
 
 # Import some modules for type-checking only.
 if TYPE_CHECKING:
-    from array import array
-    from mmap import mmap
+    from bson.typings import _DocumentType, _ReadableBuffer
 
 try:
     from bson import _cbson  # type: ignore[attr-defined]
@@ -239,8 +237,8 @@ def get_data_and_view(data: Any) -> Tuple[Any, memoryview]:
 def _raise_unknown_type(element_type: int, element_name: str) -> NoReturn:
     """Unknown type helper."""
     raise InvalidBSON(
-        "Detected unknown BSON type %r for fieldname '%s'. Are "
-        "you using the latest driver version?" % (chr(element_type).encode(), element_name)
+        "Detected unknown BSON type {!r} for fieldname '{}'. Are "
+        "you using the latest driver version?".format(chr(element_type).encode(), element_name)
     )
 
 
@@ -628,8 +626,7 @@ def gen_list_name() -> Generator[bytes, None, None]:
     The first 1000 keys are returned from a pre-built cache. All
     subsequent keys are generated on the fly.
     """
-    for name in _LIST_NAMES:
-        yield name
+    yield from _LIST_NAMES
 
     counter = itertools.count(1000)
     while True:
@@ -666,9 +663,8 @@ def _make_c_string(string: Union[str, bytes]) -> bytes:
 
 def _make_name(string: str) -> bytes:
     """Make a 'C' string suitable for a BSON key."""
-    # Keys can only be text in python 3.
     if "\x00" in string:
-        raise InvalidDocument("BSON keys / regex patterns must not contain a NUL character")
+        raise InvalidDocument("BSON keys must not contain a NUL character")
     return _utf_8_encode(string)[0] + b"\x00"
 
 
@@ -817,7 +813,7 @@ def _encode_timestamp(name: bytes, value: Any, dummy0: Any, dummy1: Any) -> byte
 
 
 def _encode_long(name: bytes, value: Any, dummy0: Any, dummy1: Any) -> bytes:
-    """Encode a python long (python 2.x)"""
+    """Encode a bson.int64.Int64."""
     try:
         return b"\x12" + name + _PACK_LONG(value)
     except struct.error:
@@ -899,12 +895,21 @@ def _name_value_to_bson(
     in_fallback_call: bool = False,
 ) -> bytes:
     """Encode a single name, value pair."""
+
+    was_integer_overflow = False
+
     # First see if the type is already cached. KeyError will only ever
     # happen once per subtype.
     try:
         return _ENCODERS[type(value)](name, value, check_keys, opts)  # type: ignore
     except KeyError:
         pass
+    except OverflowError:
+        if not isinstance(value, int):
+            raise
+
+        # Give the fallback_encoder a chance
+        was_integer_overflow = True
 
     # Second, fall back to trying _type_marker. This has to be done
     # before the loop below since users could subclass one of our
@@ -930,7 +935,7 @@ def _name_value_to_bson(
     # is done after trying the custom type encoder because checking for each
     # subtype is expensive.
     for base in _BUILT_IN_TYPES:
-        if isinstance(value, base):
+        if not was_integer_overflow and isinstance(value, base):
             func = _ENCODERS[base]
             # Cache this type for faster subsequent lookup.
             _ENCODERS[type(value)] = func
@@ -944,18 +949,20 @@ def _name_value_to_bson(
             name, fallback_encoder(value), check_keys, opts, in_fallback_call=True
         )
 
-    raise InvalidDocument("cannot encode object: %r, of type: %r" % (value, type(value)))
+    if was_integer_overflow:
+        raise OverflowError("BSON can only handle up to 8-byte ints")
+    raise InvalidDocument(f"cannot encode object: {value!r}, of type: {type(value)!r}")
 
 
 def _element_to_bson(key: Any, value: Any, check_keys: bool, opts: CodecOptions) -> bytes:
     """Encode a single key, value pair."""
     if not isinstance(key, str):
-        raise InvalidDocument("documents must have only string keys, key was %r" % (key,))
+        raise InvalidDocument(f"documents must have only string keys, key was {key!r}")
     if check_keys:
         if key.startswith("$"):
-            raise InvalidDocument("key %r must not start with '$'" % (key,))
+            raise InvalidDocument(f"key {key!r} must not start with '$'")
         if "." in key:
-            raise InvalidDocument("key %r must not contain '.'" % (key,))
+            raise InvalidDocument(f"key {key!r} must not contain '.'")
 
     name = _make_name(key)
     return _name_value_to_bson(name, value, check_keys, opts)
@@ -973,7 +980,7 @@ def _dict_to_bson(doc: Any, check_keys: bool, opts: CodecOptions, top_level: boo
             if not top_level or key != "_id":
                 elements.append(_element_to_bson(key, value, check_keys, opts))
     except AttributeError:
-        raise TypeError("encoder expected a mapping type but got: %r" % (doc,))
+        raise TypeError(f"encoder expected a mapping type but got: {doc!r}")
 
     encoded = b"".join(elements)
     return _PACK_INT(len(encoded) + 5) + encoded + b"\x00"
@@ -986,12 +993,8 @@ if _USE_C:
 _CODEC_OPTIONS_TYPE_ERROR = TypeError("codec_options must be an instance of CodecOptions")
 
 
-_DocumentIn = Mapping[str, Any]
-_ReadableBuffer = Union[bytes, memoryview, "mmap", "array"]
-
-
 def encode(
-    document: _DocumentIn,
+    document: Mapping[str, Any],
     check_keys: bool = False,
     codec_options: CodecOptions = DEFAULT_CODEC_OPTIONS,
 ) -> bytes:
@@ -1000,8 +1003,7 @@ def encode(
     A document can be any mapping type (like :class:`dict`).
 
     Raises :class:`TypeError` if `document` is not a mapping type,
-    or contains keys that are not instances of
-    :class:`basestring` (:class:`str` in python 3). Raises
+    or contains keys that are not instances of :class:`str`. Raises
     :class:`~bson.errors.InvalidDocument` if `document` cannot be
     converted to :class:`BSON`.
 
@@ -1021,9 +1023,21 @@ def encode(
     return _dict_to_bson(document, check_keys, codec_options)
 
 
+@overload
+def decode(data: "_ReadableBuffer", codec_options: None = None) -> Dict[str, Any]:
+    ...
+
+
+@overload
 def decode(
-    data: _ReadableBuffer, codec_options: "Optional[CodecOptions[_DocumentType]]" = None
-) -> _DocumentType:
+    data: "_ReadableBuffer", codec_options: "CodecOptions[_DocumentType]"
+) -> "_DocumentType":
+    ...
+
+
+def decode(
+    data: "_ReadableBuffer", codec_options: "Optional[CodecOptions[_DocumentType]]" = None
+) -> Union[Dict[str, Any], "_DocumentType"]:
     """Decode BSON to a document.
 
     By default, returns a BSON document represented as a Python
@@ -1056,11 +1070,13 @@ def decode(
     return _bson_to_dict(data, opts)
 
 
-def _decode_all(data: _ReadableBuffer, opts: "CodecOptions[_DocumentType]") -> List[_DocumentType]:
+def _decode_all(
+    data: "_ReadableBuffer", opts: "CodecOptions[_DocumentType]"
+) -> "List[_DocumentType]":
     """Decode a BSON data to multiple documents."""
     data, view = get_data_and_view(data)
     data_len = len(data)
-    docs: List[_DocumentType] = []
+    docs: "List[_DocumentType]" = []
     position = 0
     end = data_len - 1
     use_raw = _raw_document_class(opts.document_class)
@@ -1090,9 +1106,21 @@ if _USE_C:
     _decode_all = _cbson._decode_all  # noqa: F811
 
 
+@overload
+def decode_all(data: "_ReadableBuffer", codec_options: None = None) -> "List[Dict[str, Any]]":
+    ...
+
+
+@overload
 def decode_all(
-    data: _ReadableBuffer, codec_options: "Optional[CodecOptions[_DocumentType]]" = None
-) -> List[_DocumentType]:
+    data: "_ReadableBuffer", codec_options: "CodecOptions[_DocumentType]"
+) -> "List[_DocumentType]":
+    ...
+
+
+def decode_all(
+    data: "_ReadableBuffer", codec_options: "Optional[CodecOptions[_DocumentType]]" = None
+) -> "Union[List[Dict[str, Any]], List[_DocumentType]]":
     """Decode BSON data to multiple documents.
 
     `data` must be a bytes-like object implementing the buffer protocol that
@@ -1115,11 +1143,13 @@ def decode_all(
        Replaced `as_class`, `tz_aware`, and `uuid_subtype` options with
        `codec_options`.
     """
-    opts = codec_options or DEFAULT_CODEC_OPTIONS
-    if not isinstance(opts, CodecOptions):
+    if codec_options is None:
+        return _decode_all(data, DEFAULT_CODEC_OPTIONS)
+
+    if not isinstance(codec_options, CodecOptions):
         raise _CODEC_OPTIONS_TYPE_ERROR
 
-    return _decode_all(data, opts)  # type: ignore[arg-type]
+    return _decode_all(data, codec_options)
 
 
 def _decode_selective(rawdoc: Any, fields: Any, codec_options: Any) -> Mapping[Any, Any]:
@@ -1213,7 +1243,7 @@ def _decode_all_selective(data: Any, codec_options: CodecOptions, fields: Any) -
     # Decode documents for internal use.
     from bson.raw_bson import RawBSONDocument
 
-    internal_codec_options = codec_options.with_options(
+    internal_codec_options: CodecOptions[RawBSONDocument] = codec_options.with_options(
         document_class=RawBSONDocument, type_registry=None
     )
     _doc = _bson_to_dict(data, internal_codec_options)
@@ -1226,9 +1256,21 @@ def _decode_all_selective(data: Any, codec_options: CodecOptions, fields: Any) -
     ]
 
 
+@overload
+def decode_iter(data: bytes, codec_options: None = None) -> "Iterator[Dict[str, Any]]":
+    ...
+
+
+@overload
+def decode_iter(
+    data: bytes, codec_options: "CodecOptions[_DocumentType]"
+) -> "Iterator[_DocumentType]":
+    ...
+
+
 def decode_iter(
     data: bytes, codec_options: "Optional[CodecOptions[_DocumentType]]" = None
-) -> Iterator[_DocumentType]:
+) -> "Union[Iterator[Dict[str, Any]], Iterator[_DocumentType]]":
     """Decode BSON data to multiple documents as a generator.
 
     Works similarly to the decode_all function, but yields one document at a
@@ -1262,9 +1304,23 @@ def decode_iter(
         yield _bson_to_dict(elements, opts)
 
 
+@overload
+def decode_file_iter(
+    file_obj: Union[BinaryIO, IO], codec_options: None = None
+) -> "Iterator[Dict[str, Any]]":
+    ...
+
+
+@overload
+def decode_file_iter(
+    file_obj: Union[BinaryIO, IO], codec_options: "CodecOptions[_DocumentType]"
+) -> "Iterator[_DocumentType]":
+    ...
+
+
 def decode_file_iter(
     file_obj: Union[BinaryIO, IO], codec_options: "Optional[CodecOptions[_DocumentType]]" = None
-) -> Iterator[_DocumentType]:
+) -> "Union[Iterator[Dict[str, Any]], Iterator[_DocumentType]]":
     """Decode bson data from a file to multiple documents as a generator.
 
     Works similarly to the decode_all function, but reads from the file object
@@ -1286,7 +1342,7 @@ def decode_file_iter(
         # Read size of next object.
         size_data = file_obj.read(4)
         if not size_data:
-            break  # Finished with file normaly.
+            break  # Finished with file normally.
         elif len(size_data) != 4:
             raise InvalidBSON("cut off in middle of objsize")
         obj_size = _UNPACK_INT_FROM(size_data, 0)[0] - 4
@@ -1298,7 +1354,7 @@ def is_valid(bson: bytes) -> bool:
     """Check that the given string represents valid :class:`BSON` data.
 
     Raises :class:`TypeError` if `bson` is not an instance of
-    :class:`str` (:class:`bytes` in python 3). Returns ``True``
+    :class:`bytes`. Returns ``True``
     if `bson` is valid :class:`BSON`, ``False`` otherwise.
 
     :Parameters:
@@ -1325,7 +1381,7 @@ class BSON(bytes):
     @classmethod
     def encode(
         cls: Type["BSON"],
-        document: _DocumentIn,
+        document: Mapping[str, Any],
         check_keys: bool = False,
         codec_options: CodecOptions = DEFAULT_CODEC_OPTIONS,
     ) -> "BSON":
@@ -1335,9 +1391,8 @@ class BSON(bytes):
 
         Raises :class:`TypeError` if `document` is not a mapping type,
         or contains keys that are not instances of
-        :class:`basestring` (:class:`str` in python 3). Raises
-        :class:`~bson.errors.InvalidDocument` if `document` cannot be
-        converted to :class:`BSON`.
+        :class:`str'. Raises :class:`~bson.errors.InvalidDocument`
+        if `document` cannot be converted to :class:`BSON`.
 
         :Parameters:
           - `document`: mapping type representing a document
@@ -1352,7 +1407,7 @@ class BSON(bytes):
         """
         return cls(encode(document, check_keys, codec_options))
 
-    def decode(self, codec_options: "CodecOptions[_DocumentType]" = DEFAULT_CODEC_OPTIONS) -> _DocumentType:  # type: ignore[override,assignment]
+    def decode(self, codec_options: "CodecOptions[_DocumentType]" = DEFAULT_CODEC_OPTIONS) -> "_DocumentType":  # type: ignore[override,assignment]
         """Decode this BSON data.
 
         By default, returns a BSON document represented as a Python
@@ -1391,7 +1446,7 @@ def has_c() -> bool:
     return _USE_C
 
 
-def _after_fork():
+def _after_fork() -> None:
     """Releases the ObjectID lock child."""
     if ObjectId._inc_lock.locked():
         ObjectId._inc_lock.release()
