@@ -29,7 +29,9 @@ Example usage (deserialization):
 .. doctest::
 
    >>> from bson.json_util import loads
-   >>> loads('[{"foo": [1, 2]}, {"bar": {"hello": "world"}}, {"code": {"$scope": {}, "$code": "function x() { return 1; }"}}, {"bin": {"$type": "80", "$binary": "AQIDBA=="}}]')
+   >>> loads(
+   ...     '[{"foo": [1, 2]}, {"bar": {"hello": "world"}}, {"code": {"$scope": {}, "$code": "function x() { return 1; }"}}, {"bin": {"$type": "80", "$binary": "AQIDBA=="}}]'
+   ... )
    [{'foo': [1, 2]}, {'bar': {'hello': 'world'}}, {'code': Code('function x() { return 1; }', {})}, {'bin': Binary(b'...', 128)}]
 
 Example usage with :const:`RELAXED_JSON_OPTIONS` (the default):
@@ -38,10 +40,14 @@ Example usage with :const:`RELAXED_JSON_OPTIONS` (the default):
 
    >>> from bson import Binary, Code
    >>> from bson.json_util import dumps
-   >>> dumps([{'foo': [1, 2]},
-   ...        {'bar': {'hello': 'world'}},
-   ...        {'code': Code("function x() { return 1; }")},
-   ...        {'bin': Binary(b"\x01\x02\x03\x04")}])
+   >>> dumps(
+   ...     [
+   ...         {"foo": [1, 2]},
+   ...         {"bar": {"hello": "world"}},
+   ...         {"code": Code("function x() { return 1; }")},
+   ...         {"bin": Binary(b"\x01\x02\x03\x04")},
+   ...     ]
+   ... )
    '[{"foo": [1, 2]}, {"bar": {"hello": "world"}}, {"code": {"$code": "function x() { return 1; }"}}, {"bin": {"$binary": {"base64": "AQIDBA==", "subType": "00"}}}]'
 
 Example usage (with :const:`CANONICAL_JSON_OPTIONS`):
@@ -50,11 +56,15 @@ Example usage (with :const:`CANONICAL_JSON_OPTIONS`):
 
    >>> from bson import Binary, Code
    >>> from bson.json_util import dumps, CANONICAL_JSON_OPTIONS
-   >>> dumps([{'foo': [1, 2]},
-   ...        {'bar': {'hello': 'world'}},
-   ...        {'code': Code("function x() { return 1; }")},
-   ...        {'bin': Binary(b"\x01\x02\x03\x04")}],
-   ...       json_options=CANONICAL_JSON_OPTIONS)
+   >>> dumps(
+   ...     [
+   ...         {"foo": [1, 2]},
+   ...         {"bar": {"hello": "world"}},
+   ...         {"code": Code("function x() { return 1; }")},
+   ...         {"bin": Binary(b"\x01\x02\x03\x04")},
+   ...     ],
+   ...     json_options=CANONICAL_JSON_OPTIONS,
+   ... )
    '[{"foo": [{"$numberInt": "1"}, {"$numberInt": "2"}]}, {"bar": {"hello": "world"}}, {"code": {"$code": "function x() { return 1; }"}}, {"bin": {"$binary": {"base64": "AQIDBA==", "subType": "00"}}}]'
 
 Example usage (with :const:`LEGACY_JSON_OPTIONS`):
@@ -63,11 +73,15 @@ Example usage (with :const:`LEGACY_JSON_OPTIONS`):
 
    >>> from bson import Binary, Code
    >>> from bson.json_util import dumps, LEGACY_JSON_OPTIONS
-   >>> dumps([{'foo': [1, 2]},
-   ...        {'bar': {'hello': 'world'}},
-   ...        {'code': Code("function x() { return 1; }", {})},
-   ...        {'bin': Binary(b"\x01\x02\x03\x04")}],
-   ...       json_options=LEGACY_JSON_OPTIONS)
+   >>> dumps(
+   ...     [
+   ...         {"foo": [1, 2]},
+   ...         {"bar": {"hello": "world"}},
+   ...         {"code": Code("function x() { return 1; }", {})},
+   ...         {"bin": Binary(b"\x01\x02\x03\x04")},
+   ...     ],
+   ...     json_options=LEGACY_JSON_OPTIONS,
+   ... )
    '[{"foo": [1, 2]}, {"bar": {"hello": "world"}}, {"code": {"$code": "function x() { return 1; }", "$scope": {}}}, {"bin": {"$binary": "AQIDBA==", "$type": "00"}}]'
 
 Alternatively, you can manually pass the `default` to :func:`json.dumps`.
@@ -205,66 +219,68 @@ class JSONMode:
 
 
 class JSONOptions(CodecOptions):
-    """Encapsulates JSON options for :func:`dumps` and :func:`loads`.
-
-    :Parameters:
-      - `strict_number_long`: If ``True``, :class:`~bson.int64.Int64` objects
-        are encoded to MongoDB Extended JSON's *Strict mode* type
-        `NumberLong`, ie ``'{"$numberLong": "<number>" }'``. Otherwise they
-        will be encoded as an `int`. Defaults to ``False``.
-      - `datetime_representation`: The representation to use when encoding
-        instances of :class:`datetime.datetime`. Defaults to
-        :const:`~DatetimeRepresentation.LEGACY`.
-      - `strict_uuid`: If ``True``, :class:`uuid.UUID` object are encoded to
-        MongoDB Extended JSON's *Strict mode* type `Binary`. Otherwise it
-        will be encoded as ``'{"$uuid": "<hex>" }'``. Defaults to ``False``.
-      - `json_mode`: The :class:`JSONMode` to use when encoding BSON types to
-        Extended JSON. Defaults to :const:`~JSONMode.LEGACY`.
-      - `document_class`: BSON documents returned by :func:`loads` will be
-        decoded to an instance of this class. Must be a subclass of
-        :class:`collections.MutableMapping`. Defaults to :class:`dict`.
-      - `uuid_representation`: The :class:`~bson.binary.UuidRepresentation`
-        to use when encoding and decoding instances of :class:`uuid.UUID`.
-        Defaults to :const:`~bson.binary.UuidRepresentation.UNSPECIFIED`.
-      - `tz_aware`: If ``True``, MongoDB Extended JSON's *Strict mode* type
-        `Date` will be decoded to timezone aware instances of
-        :class:`datetime.datetime`. Otherwise they will be naive. Defaults
-        to ``False``.
-      - `tzinfo`: A :class:`datetime.tzinfo` subclass that specifies the
-        timezone from which :class:`~datetime.datetime` objects should be
-        decoded. Defaults to :const:`~bson.tz_util.utc`.
-      - `datetime_conversion`: Specifies how UTC datetimes should be decoded
-        within BSON. Valid options include 'datetime_ms' to return as a
-        DatetimeMS, 'datetime' to return as a datetime.datetime and
-        raising a ValueError for out-of-range values, 'datetime_auto' to
-        return DatetimeMS objects when the underlying datetime is
-        out-of-range and 'datetime_clamp' to clamp to the minimum and
-        maximum possible datetimes. Defaults to 'datetime'. See
-        :ref:`handling-out-of-range-datetimes` for details.
-      - `args`: arguments to :class:`~bson.codec_options.CodecOptions`
-      - `kwargs`: arguments to :class:`~bson.codec_options.CodecOptions`
-
-    .. seealso:: The specification for Relaxed and Canonical `Extended JSON`_.
-
-    .. versionchanged:: 4.0
-       The default for `json_mode` was changed from :const:`JSONMode.LEGACY`
-       to :const:`JSONMode.RELAXED`.
-       The default for `uuid_representation` was changed from
-       :const:`~bson.binary.UuidRepresentation.PYTHON_LEGACY` to
-       :const:`~bson.binary.UuidRepresentation.UNSPECIFIED`.
-
-    .. versionchanged:: 3.5
-       Accepts the optional parameter `json_mode`.
-
-    .. versionchanged:: 4.0
-       Changed default value of `tz_aware` to False.
-    """
-
     json_mode: int
     strict_number_long: bool
     datetime_representation: int
     strict_uuid: bool
 
+    def __init__(self, *args: Any, **kwargs: Any):
+        """Encapsulates JSON options for :func:`dumps` and :func:`loads`.
+
+        :Parameters:
+          - `strict_number_long`: If ``True``, :class:`~bson.int64.Int64` objects
+            are encoded to MongoDB Extended JSON's *Strict mode* type
+            `NumberLong`, ie ``'{"$numberLong": "<number>" }'``. Otherwise they
+            will be encoded as an `int`. Defaults to ``False``.
+          - `datetime_representation`: The representation to use when encoding
+            instances of :class:`datetime.datetime`. Defaults to
+            :const:`~DatetimeRepresentation.LEGACY`.
+          - `strict_uuid`: If ``True``, :class:`uuid.UUID` object are encoded to
+            MongoDB Extended JSON's *Strict mode* type `Binary`. Otherwise it
+            will be encoded as ``'{"$uuid": "<hex>" }'``. Defaults to ``False``.
+          - `json_mode`: The :class:`JSONMode` to use when encoding BSON types to
+            Extended JSON. Defaults to :const:`~JSONMode.LEGACY`.
+          - `document_class`: BSON documents returned by :func:`loads` will be
+            decoded to an instance of this class. Must be a subclass of
+            :class:`collections.MutableMapping`. Defaults to :class:`dict`.
+          - `uuid_representation`: The :class:`~bson.binary.UuidRepresentation`
+            to use when encoding and decoding instances of :class:`uuid.UUID`.
+            Defaults to :const:`~bson.binary.UuidRepresentation.UNSPECIFIED`.
+          - `tz_aware`: If ``True``, MongoDB Extended JSON's *Strict mode* type
+            `Date` will be decoded to timezone aware instances of
+            :class:`datetime.datetime`. Otherwise they will be naive. Defaults
+            to ``False``.
+          - `tzinfo`: A :class:`datetime.tzinfo` subclass that specifies the
+            timezone from which :class:`~datetime.datetime` objects should be
+            decoded. Defaults to :const:`~bson.tz_util.utc`.
+          - `datetime_conversion`: Specifies how UTC datetimes should be decoded
+            within BSON. Valid options include 'datetime_ms' to return as a
+            DatetimeMS, 'datetime' to return as a datetime.datetime and
+            raising a ValueError for out-of-range values, 'datetime_auto' to
+            return DatetimeMS objects when the underlying datetime is
+            out-of-range and 'datetime_clamp' to clamp to the minimum and
+            maximum possible datetimes. Defaults to 'datetime'. See
+            :ref:`handling-out-of-range-datetimes` for details.
+          - `args`: arguments to :class:`~bson.codec_options.CodecOptions`
+          - `kwargs`: arguments to :class:`~bson.codec_options.CodecOptions`
+
+        .. seealso:: The specification for Relaxed and Canonical `Extended JSON`_.
+
+        .. versionchanged:: 4.0
+           The default for `json_mode` was changed from :const:`JSONMode.LEGACY`
+           to :const:`JSONMode.RELAXED`.
+           The default for `uuid_representation` was changed from
+           :const:`~bson.binary.UuidRepresentation.PYTHON_LEGACY` to
+           :const:`~bson.binary.UuidRepresentation.UNSPECIFIED`.
+
+        .. versionchanged:: 3.5
+           Accepts the optional parameter `json_mode`.
+
+        .. versionchanged:: 4.0
+           Changed default value of `tz_aware` to False.
+        """
+        super().__init__()
+
     def __new__(
         cls: Type["JSONOptions"],
         strict_number_long: Optional[bool] = None,
@@ -272,7 +288,7 @@ class JSONOptions(CodecOptions):
         strict_uuid: Optional[bool] = None,
         json_mode: int = JSONMode.RELAXED,
         *args: Any,
-        **kwargs: Any
+        **kwargs: Any,
     ) -> "JSONOptions":
         kwargs["tz_aware"] = kwargs.get("tz_aware", False)
         if kwargs["tz_aware"]:
@@ -287,7 +303,7 @@ class JSONOptions(CodecOptions):
                 "JSONOptions.datetime_representation must be one of LEGACY, "
                 "NUMBERLONG, or ISO8601 from DatetimeRepresentation."
             )
-        self = cast(JSONOptions, super(JSONOptions, cls).__new__(cls, *args, **kwargs))
+        self = cast(JSONOptions, super().__new__(cls, *args, **kwargs))
         if json_mode not in (JSONMode.LEGACY, JSONMode.RELAXED, JSONMode.CANONICAL):
             raise ValueError(
                 "JSONOptions.json_mode must be one of LEGACY, RELAXED, "
@@ -334,21 +350,20 @@ class JSONOptions(CodecOptions):
 
     def _arguments_repr(self) -> str:
         return (
-            "strict_number_long=%r, "
-            "datetime_representation=%r, "
-            "strict_uuid=%r, json_mode=%r, %s"
-            % (
+            "strict_number_long={!r}, "
+            "datetime_representation={!r}, "
+            "strict_uuid={!r}, json_mode={!r}, {}".format(
                 self.strict_number_long,
                 self.datetime_representation,
                 self.strict_uuid,
                 self.json_mode,
-                super(JSONOptions, self)._arguments_repr(),
+                super()._arguments_repr(),
             )
         )
 
     def _options_dict(self) -> Dict[Any, Any]:
         # TODO: PYTHON-2442 use _asdict() instead
-        options_dict = super(JSONOptions, self)._options_dict()
+        options_dict = super()._options_dict()
         options_dict.update(
             {
                 "strict_number_long": self.strict_number_long,
@@ -438,7 +453,7 @@ def dumps(obj: Any, *args: Any, **kwargs: Any) -> str:
     return json.dumps(_json_convert(obj, json_options), *args, **kwargs)
 
 
-def loads(s: str, *args: Any, **kwargs: Any) -> Any:
+def loads(s: Union[str, bytes, bytearray], *args: Any, **kwargs: Any) -> Any:
     """Helper function that wraps :func:`json.loads`.
 
     Automatically passes the object_hook for BSON type conversion.
@@ -476,7 +491,7 @@ def _json_convert(obj: Any, json_options: JSONOptions = DEFAULT_JSON_OPTIONS) ->
     if hasattr(obj, "items"):
         return SON(((k, _json_convert(v, json_options)) for k, v in obj.items()))
     elif hasattr(obj, "__iter__") and not isinstance(obj, (str, bytes)):
-        return list((_json_convert(v, json_options) for v in obj))
+        return [_json_convert(v, json_options) for v in obj]
     try:
         return default(obj, json_options)
     except TypeError:
@@ -552,9 +567,9 @@ def _parse_legacy_regex(doc: Any) -> Any:
 def _parse_legacy_uuid(doc: Any, json_options: JSONOptions) -> Union[Binary, uuid.UUID]:
     """Decode a JSON legacy $uuid to Python UUID."""
     if len(doc) != 1:
-        raise TypeError("Bad $uuid, extra field(s): %s" % (doc,))
+        raise TypeError(f"Bad $uuid, extra field(s): {doc}")
     if not isinstance(doc["$uuid"], str):
-        raise TypeError("$uuid must be a string: %s" % (doc,))
+        raise TypeError(f"$uuid must be a string: {doc}")
     if json_options.uuid_representation == UuidRepresentation.UNSPECIFIED:
         return Binary.from_uuid(uuid.UUID(doc["$uuid"]))
     else:
@@ -597,11 +612,11 @@ def _parse_canonical_binary(doc: Any, json_options: JSONOptions) -> Union[Binary
     b64 = binary["base64"]
     subtype = binary["subType"]
     if not isinstance(b64, str):
-        raise TypeError("$binary base64 must be a string: %s" % (doc,))
+        raise TypeError(f"$binary base64 must be a string: {doc}")
     if not isinstance(subtype, str) or len(subtype) > 2:
-        raise TypeError("$binary subType must be a string at most 2 characters: %s" % (doc,))
+        raise TypeError(f"$binary subType must be a string at most 2 characters: {doc}")
     if len(binary) != 2:
-        raise TypeError('$binary must include only "base64" and "subType" components: %s' % (doc,))
+        raise TypeError(f'$binary must include only "base64" and "subType" components: {doc}')
 
     data = base64.b64decode(b64.encode())
     return _binary_or_uuid(data, int(subtype, 16), json_options)
@@ -613,7 +628,7 @@ def _parse_canonical_datetime(
     """Decode a JSON datetime to python datetime.datetime."""
     dtm = doc["$date"]
     if len(doc) != 1:
-        raise TypeError("Bad $date, extra field(s): %s" % (doc,))
+        raise TypeError(f"Bad $date, extra field(s): {doc}")
     # mongoexport 2.6 and newer
     if isinstance(dtm, str):
         # Parse offset
@@ -676,7 +691,7 @@ def _parse_canonical_datetime(
 def _parse_canonical_oid(doc: Any) -> ObjectId:
     """Decode a JSON ObjectId to bson.objectid.ObjectId."""
     if len(doc) != 1:
-        raise TypeError("Bad $oid, extra field(s): %s" % (doc,))
+        raise TypeError(f"Bad $oid, extra field(s): {doc}")
     return ObjectId(doc["$oid"])
 
 
@@ -684,7 +699,7 @@ def _parse_canonical_symbol(doc: Any) -> str:
     """Decode a JSON symbol to Python string."""
     symbol = doc["$symbol"]
     if len(doc) != 1:
-        raise TypeError("Bad $symbol, extra field(s): %s" % (doc,))
+        raise TypeError(f"Bad $symbol, extra field(s): {doc}")
     return str(symbol)
 
 
@@ -692,7 +707,7 @@ def _parse_canonical_code(doc: Any) -> Code:
     """Decode a JSON code to bson.code.Code."""
     for key in doc:
         if key not in ("$code", "$scope"):
-            raise TypeError("Bad $code, extra field(s): %s" % (doc,))
+            raise TypeError(f"Bad $code, extra field(s): {doc}")
     return Code(doc["$code"], scope=doc.get("$scope"))
 
 
@@ -700,11 +715,11 @@ def _parse_canonical_regex(doc: Any) -> Regex:
     """Decode a JSON regex to bson.regex.Regex."""
     regex = doc["$regularExpression"]
     if len(doc) != 1:
-        raise TypeError("Bad $regularExpression, extra field(s): %s" % (doc,))
+        raise TypeError(f"Bad $regularExpression, extra field(s): {doc}")
     if len(regex) != 2:
         raise TypeError(
             'Bad $regularExpression must include only "pattern"'
-            'and "options" components: %s' % (doc,)
+            'and "options" components: {}'.format(doc)
         )
     opts = regex["options"]
     if not isinstance(opts, str):
@@ -723,28 +738,28 @@ def _parse_canonical_dbpointer(doc: Any) -> Any:
     """Decode a JSON (deprecated) DBPointer to bson.dbref.DBRef."""
     dbref = doc["$dbPointer"]
     if len(doc) != 1:
-        raise TypeError("Bad $dbPointer, extra field(s): %s" % (doc,))
+        raise TypeError(f"Bad $dbPointer, extra field(s): {doc}")
     if isinstance(dbref, DBRef):
         dbref_doc = dbref.as_doc()
         # DBPointer must not contain $db in its value.
         if dbref.database is not None:
-            raise TypeError("Bad $dbPointer, extra field $db: %s" % (dbref_doc,))
+            raise TypeError(f"Bad $dbPointer, extra field $db: {dbref_doc}")
         if not isinstance(dbref.id, ObjectId):
-            raise TypeError("Bad $dbPointer, $id must be an ObjectId: %s" % (dbref_doc,))
+            raise TypeError(f"Bad $dbPointer, $id must be an ObjectId: {dbref_doc}")
         if len(dbref_doc) != 2:
-            raise TypeError("Bad $dbPointer, extra field(s) in DBRef: %s" % (dbref_doc,))
+            raise TypeError(f"Bad $dbPointer, extra field(s) in DBRef: {dbref_doc}")
         return dbref
     else:
-        raise TypeError("Bad $dbPointer, expected a DBRef: %s" % (doc,))
+        raise TypeError(f"Bad $dbPointer, expected a DBRef: {doc}")
 
 
 def _parse_canonical_int32(doc: Any) -> int:
     """Decode a JSON int32 to python int."""
     i_str = doc["$numberInt"]
     if len(doc) != 1:
-        raise TypeError("Bad $numberInt, extra field(s): %s" % (doc,))
+        raise TypeError(f"Bad $numberInt, extra field(s): {doc}")
     if not isinstance(i_str, str):
-        raise TypeError("$numberInt must be string: %s" % (doc,))
+        raise TypeError(f"$numberInt must be string: {doc}")
     return int(i_str)
 
 
@@ -752,7 +767,7 @@ def _parse_canonical_int64(doc: Any) -> Int64:
     """Decode a JSON int64 to bson.int64.Int64."""
     l_str = doc["$numberLong"]
     if len(doc) != 1:
-        raise TypeError("Bad $numberLong, extra field(s): %s" % (doc,))
+        raise TypeError(f"Bad $numberLong, extra field(s): {doc}")
     return Int64(l_str)
 
 
@@ -760,9 +775,9 @@ def _parse_canonical_double(doc: Any) -> float:
     """Decode a JSON double to python float."""
     d_str = doc["$numberDouble"]
     if len(doc) != 1:
-        raise TypeError("Bad $numberDouble, extra field(s): %s" % (doc,))
+        raise TypeError(f"Bad $numberDouble, extra field(s): {doc}")
     if not isinstance(d_str, str):
-        raise TypeError("$numberDouble must be string: %s" % (doc,))
+        raise TypeError(f"$numberDouble must be string: {doc}")
     return float(d_str)
 
 
@@ -770,18 +785,18 @@ def _parse_canonical_decimal128(doc: Any) -> Decimal128:
     """Decode a JSON decimal128 to bson.decimal128.Decimal128."""
     d_str = doc["$numberDecimal"]
     if len(doc) != 1:
-        raise TypeError("Bad $numberDecimal, extra field(s): %s" % (doc,))
+        raise TypeError(f"Bad $numberDecimal, extra field(s): {doc}")
     if not isinstance(d_str, str):
-        raise TypeError("$numberDecimal must be string: %s" % (doc,))
+        raise TypeError(f"$numberDecimal must be string: {doc}")
     return Decimal128(d_str)
 
 
 def _parse_canonical_minkey(doc: Any) -> MinKey:
     """Decode a JSON MinKey to bson.min_key.MinKey."""
     if type(doc["$minKey"]) is not int or doc["$minKey"] != 1:
-        raise TypeError("$minKey value must be 1: %s" % (doc,))
+        raise TypeError(f"$minKey value must be 1: {doc}")
     if len(doc) != 1:
-        raise TypeError("Bad $minKey, extra field(s): %s" % (doc,))
+        raise TypeError(f"Bad $minKey, extra field(s): {doc}")
     return MinKey()
 
 
@@ -790,7 +805,7 @@ def _parse_canonical_maxkey(doc: Any) -> MaxKey:
     if type(doc["$maxKey"]) is not int or doc["$maxKey"] != 1:
         raise TypeError("$maxKey value must be 1: %s", (doc,))
     if len(doc) != 1:
-        raise TypeError("Bad $minKey, extra field(s): %s" % (doc,))
+        raise TypeError(f"Bad $minKey, extra field(s): {doc}")
     return MaxKey()
 
 
@@ -823,7 +838,7 @@ def default(obj: Any, json_options: JSONOptions = DEFAULT_JSON_OPTIONS) -> Any:
                 millis = int(obj.microsecond / 1000)
                 fracsecs = ".%03d" % (millis,) if millis else ""
                 return {
-                    "$date": "%s%s%s" % (obj.strftime("%Y-%m-%dT%H:%M:%S"), fracsecs, tz_string)
+                    "$date": "{}{}{}".format(obj.strftime("%Y-%m-%dT%H:%M:%S"), fracsecs, tz_string)
                 }
 
         millis = _datetime_to_millis(obj)
