@@ -14,7 +14,7 @@
 
 """Python driver for MongoDB."""
 
-from typing import ContextManager, Optional, Tuple, Union
+from typing import ContextManager, Optional
 
 __all__ = [
     "ASCENDING",
@@ -84,21 +84,8 @@ TEXT = "text"
 .. _text index: http://mongodb.com/docs/manual/core/index-text/
 """
 
-version_tuple: Tuple[Union[int, str], ...] = (4, 2, 0)
-
-
-def get_version_string() -> str:
-    if isinstance(version_tuple[-1], str):
-        return ".".join(map(str, version_tuple[:-1])) + version_tuple[-1]
-    return ".".join(map(str, version_tuple))
-
-
-__version__: str = get_version_string()
-version = __version__
-
-"""Current version of PyMongo."""
-
 from pymongo import _csot
+from pymongo._version import __version__, get_version_string, version_tuple
 from pymongo.collection import ReturnDocument
 from pymongo.common import MAX_SUPPORTED_WIRE_VERSION, MIN_SUPPORTED_WIRE_VERSION
 from pymongo.cursor import CursorType
@@ -115,6 +102,9 @@ from pymongo.operations import (
 from pymongo.read_preferences import ReadPreference
 from pymongo.write_concern import WriteConcern
 
+version = __version__
+"""Current version of PyMongo."""
+
 
 def has_c() -> bool:
     """Is the C extension installed?"""
@@ -155,8 +145,8 @@ def timeout(seconds: Optional[float]) -> ContextManager:
           else:
               print(f"failed with non-timeout error: {exc!r}")
 
-    When nesting :func:`~pymongo.timeout`, the newly computed deadline is capped to at most
-    the existing deadline. The deadline can only be shortened, not extended.
+    When nesting :func:`~pymongo.timeout`, the nested deadline is capped by
+    the outer deadline. The deadline can only be shortened, not extended.
     When exiting the block, the previous deadline is restored::
 
       with pymongo.timeout(5):
@@ -174,6 +164,8 @@ def timeout(seconds: Optional[float]) -> ContextManager:
     :Raises:
       - :py:class:`ValueError`: When `seconds` is negative.
 
+    See :ref:`timeout-example` for more examples.
+
     .. versionadded:: 4.2
     """
     if not isinstance(seconds, (int, float, type(None))):
