@@ -13,15 +13,27 @@
 # permissions and limitations under the License.
 
 """Watch changes on a collection, a database, or the entire cluster."""
+from __future__ import annotations
 
 import copy
-from typing import TYPE_CHECKING, Any, Dict, Generic, Mapping, Optional, Union
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Dict,
+    Generic,
+    List,
+    Mapping,
+    Optional,
+    Type,
+    Union,
+)
 
 from bson import _bson_to_dict
 from bson.raw_bson import RawBSONDocument
 from bson.timestamp import Timestamp
 from pymongo import _csot, common
 from pymongo.aggregation import (
+    _AggregationCommand,
     _CollectionAggregationCommand,
     _DatabaseAggregationCommand,
 )
@@ -66,6 +78,20 @@ if TYPE_CHECKING:
     from pymongo.collection import Collection
     from pymongo.database import Database
     from pymongo.mongo_client import MongoClient
+    from pymongo.pool import Connection
+
+
+def _resumable(exc: PyMongoError) -> bool:
+    """Return True if given a resumable change stream error."""
+    if isinstance(exc, (ConnectionFailure, CursorNotFound)):
+        return True
+    if isinstance(exc, OperationFailure):
+        if exc._max_wire_version is None:
+            return False
+        return (
+            exc._max_wire_version >= 9 and exc.has_error_label("ResumableChangeStreamError")
+        ) or (exc._max_wire_version < 9 and exc.code in _RESUMABLE_GETMORE_ERRORS)
+    return False
 
 
 class ChangeStream(Generic[_DocumentType]):
@@ -83,7 +109,7 @@ class ChangeStream(Generic[_DocumentType]):
     def __init__(
         self,
         target: Union[
-            "MongoClient[_DocumentType]", "Database[_DocumentType]", "Collection[_DocumentType]"
+            MongoClient[_DocumentType], Database[_DocumentType], Collection[_DocumentType]
         ],
         pipeline: Optional[_Pipeline],
         full_document: Optional[str],
@@ -92,10 +118,11 @@ class ChangeStream(Generic[_DocumentType]):
         batch_size: Optional[int],
         collation: Optional[_CollationIn],
         start_at_operation_time: Optional[Timestamp],
-        session: Optional["ClientSession"],
+        session: Optional[ClientSession],
         start_after: Optional[Mapping[str, Any]],
         comment: Optional[Any] = None,
         full_document_before_change: Optional[str] = None,
+        show_expanded_events: Optional[bool] = None,
     ) -> None:
         if pipeline is None:
             pipeline = []
@@ -130,21 +157,23 @@ class ChangeStream(Generic[_DocumentType]):
         self._comment = comment
         self._closed = False
         self._timeout = self._target._timeout
+        self._show_expanded_events = show_expanded_events
         # Initialize cursor.
         self._cursor = self._create_cursor()
 
     @property
-    def _aggregation_command_class(self):
+    def _aggregation_command_class(self) -> Type[_AggregationCommand]:
         """The aggregation command class to be used."""
         raise NotImplementedError
 
     @property
-    def _client(self):
+    def _client(self) -> MongoClient:
         """The client against which the aggregation commands for
-        this ChangeStream will be run."""
+        this ChangeStream will be run.
+        """
         raise NotImplementedError
 
-    def _change_stream_options(self):
+    def _change_stream_options(self) -> Dict[str, Any]:
         """Return the options dict for the $changeStream pipeline stage."""
         options: Dict[str, Any] = {}
         if self._full_document is not None:
@@ -162,9 +191,13 @@ class ChangeStream(Generic[_DocumentType]):
 
         if self._start_at_operation_time is not None:
             options["startAtOperationTime"] = self._start_at_operation_time
+
+        if self._show_expanded_events:
+            options["showExpandedEvents"] = self._show_expanded_events
+
         return options
 
-    def _command_options(self):
+    def _command_options(self) -> Dict[str, Any]:
         """Return the options dict for the aggregation command."""
         options = {}
         if self._max_await_time_ms is not None:
@@ -173,14 +206,14 @@ class ChangeStream(Generic[_DocumentType]):
             options["batchSize"] = self._batch_size
         return options
 
-    def _aggregation_pipeline(self):
+    def _aggregation_pipeline(self) -> List[Dict[str, Any]]:
         """Return the full aggregation pipeline for this ChangeStream."""
         options = self._change_stream_options()
         full_pipeline: list = [{"$changeStream": options}]
         full_pipeline.extend(self._pipeline)
         return full_pipeline
 
-    def _process_result(self, result, sock_info):
+    def _process_result(self, result: Mapping[str, Any], conn: Connection) -> None:
         """Callback that caches the postBatchResumeToken or
         startAtOperationTime from a changeStream aggregate command response
         containing an empty batch of change documents.
@@ -195,17 +228,19 @@ class ChangeStream(Generic[_DocumentType]):
                 self._start_at_operation_time is None
                 and self._uses_resume_after is False
                 and self._uses_start_after is False
-                and sock_info.max_wire_version >= 7
+                and conn.max_wire_version >= 7
             ):
                 self._start_at_operation_time = result.get("operationTime")
                 # PYTHON-2181: informative error on missing operationTime.
                 if self._start_at_operation_time is None:
                     raise OperationFailure(
                         "Expected field 'operationTime' missing from command "
-                        "response : %r" % (result,)
+                        "response : {!r}".format(result)
                     )
 
-    def _run_aggregation_cmd(self, session, explicit_session):
+    def _run_aggregation_cmd(
+        self, session: Optional[ClientSession], explicit_session: bool
+    ) -> CommandCursor:
         """Run the full aggregation pipeline for this ChangeStream and return
         the corresponding CommandCursor.
         """
@@ -222,11 +257,11 @@ class ChangeStream(Generic[_DocumentType]):
             cmd.get_cursor, self._target._read_preference_for(session), session
         )
 
-    def _create_cursor(self):
+    def _create_cursor(self) -> CommandCursor:
         with self._client._tmp_session(self._session, close=False) as s:
             return self._run_aggregation_cmd(session=s, explicit_session=self._session is not None)
 
-    def _resume(self):
+    def _resume(self) -> None:
         """Reestablish this change stream after a resumable error."""
         try:
             self._cursor.close()
@@ -343,20 +378,21 @@ class ChangeStream(Generic[_DocumentType]):
         # Attempt to get the next change with at most one getMore and at most
         # one resume attempt.
         try:
-            change = self._cursor._try_next(True)
-        except (ConnectionFailure, CursorNotFound):
-            self._resume()
-            change = self._cursor._try_next(False)
-        except OperationFailure as exc:
-            if exc._max_wire_version is None:
-                raise
-            is_resumable = (
-                exc._max_wire_version >= 9 and exc.has_error_label("ResumableChangeStreamError")
-            ) or (exc._max_wire_version < 9 and exc.code in _RESUMABLE_GETMORE_ERRORS)
-            if not is_resumable:
-                raise
-            self._resume()
-            change = self._cursor._try_next(False)
+            try:
+                change = self._cursor._try_next(True)
+            except PyMongoError as exc:
+                if not _resumable(exc):
+                    raise
+                self._resume()
+                change = self._cursor._try_next(False)
+        except PyMongoError as exc:
+            # Close the stream after a fatal error.
+            if not _resumable(exc) and not exc.timeout:
+                self.close()
+            raise
+        except Exception:
+            self.close()
+            raise
 
         # Check if the cursor was invalidated.
         if not self._cursor.alive:
@@ -415,12 +451,14 @@ class CollectionChangeStream(ChangeStream, Generic[_DocumentType]):
     .. versionadded:: 3.7
     """
 
+    _target: Collection[_DocumentType]
+
     @property
-    def _aggregation_command_class(self):
+    def _aggregation_command_class(self) -> Type[_CollectionAggregationCommand]:
         return _CollectionAggregationCommand
 
     @property
-    def _client(self):
+    def _client(self) -> MongoClient:
         return self._target.database.client
 
 
@@ -433,12 +471,14 @@ class DatabaseChangeStream(ChangeStream, Generic[_DocumentType]):
     .. versionadded:: 3.7
     """
 
+    _target: Database[_DocumentType]
+
     @property
-    def _aggregation_command_class(self):
+    def _aggregation_command_class(self) -> Type[_DatabaseAggregationCommand]:
         return _DatabaseAggregationCommand
 
     @property
-    def _client(self):
+    def _client(self) -> MongoClient:
         return self._target.client
 
 
@@ -451,7 +491,7 @@ class ClusterChangeStream(DatabaseChangeStream, Generic[_DocumentType]):
     .. versionadded:: 3.7
     """
 
-    def _change_stream_options(self):
-        options = super(ClusterChangeStream, self)._change_stream_options()
+    def _change_stream_options(self) -> Dict[str, Any]:
+        options = super()._change_stream_options()
         options["allChangesForCluster"] = True
         return options
