@@ -14,14 +14,18 @@
 
 
 """Functions and classes common to multiple pymongo modules."""
+from __future__ import annotations
 
 import datetime
+import inspect
 import warnings
 from collections import OrderedDict, abc
 from typing import (
+    TYPE_CHECKING,
     Any,
     Callable,
     Dict,
+    Iterator,
     List,
     Mapping,
     MutableMapping,
@@ -31,12 +35,13 @@ from typing import (
     Tuple,
     Type,
     Union,
+    overload,
 )
 from urllib.parse import unquote_plus
 
 from bson import SON
 from bson.binary import UuidRepresentation
-from bson.codec_options import CodecOptions, TypeRegistry
+from bson.codec_options import CodecOptions, DatetimeConversion, TypeRegistry
 from bson.raw_bson import RawBSONDocument
 from pymongo.auth import MECHANISMS
 from pymongo.compression_support import (
@@ -49,7 +54,11 @@ from pymongo.monitoring import _validate_event_listeners
 from pymongo.read_concern import ReadConcern
 from pymongo.read_preferences import _MONGOS_MODES, _ServerMode
 from pymongo.server_api import ServerApi
-from pymongo.write_concern import DEFAULT_WRITE_CONCERN, WriteConcern
+from pymongo.write_concern import DEFAULT_WRITE_CONCERN, WriteConcern, validate_boolean
+
+if TYPE_CHECKING:
+    from pymongo.client_session import ClientSession
+
 
 ORDERED_TYPES: Sequence[Type] = (SON, OrderedDict)
 
@@ -63,7 +72,7 @@ MAX_WRITE_BATCH_SIZE = 1000
 # What this version of PyMongo supports.
 MIN_SUPPORTED_SERVER_VERSION = "3.6"
 MIN_SUPPORTED_WIRE_VERSION = 6
-MAX_SUPPORTED_WIRE_VERSION = 17
+MAX_SUPPORTED_WIRE_VERSION = 21
 
 # Frequency to call hello on servers, in seconds.
 HEARTBEAT_FREQUENCY = 10
@@ -156,7 +165,7 @@ def clean_node(node: str) -> Tuple[str, int]:
 
 def raise_config_error(key: str, dummy: Any) -> NoReturn:
     """Raise ConfigurationError with the given key name."""
-    raise ConfigurationError("Unknown option %s" % (key,))
+    raise ConfigurationError(f"Unknown option {key}")
 
 
 # Mapping of URI uuid representation options to valid subtypes.
@@ -169,18 +178,11 @@ _UUID_REPRESENTATIONS = {
 }
 
 
-def validate_boolean(option: str, value: Any) -> bool:
-    """Validates that 'value' is True or False."""
-    if isinstance(value, bool):
-        return value
-    raise TypeError("%s must be True or False" % (option,))
-
-
 def validate_boolean_or_string(option: str, value: Any) -> bool:
     """Validates that value is True, False, 'true', or 'false'."""
     if isinstance(value, str):
         if value not in ("true", "false"):
-            raise ValueError("The value of %s must be 'true' or 'false'" % (option,))
+            raise ValueError(f"The value of {option} must be 'true' or 'false'")
         return value == "true"
     return validate_boolean(option, value)
 
@@ -193,15 +195,15 @@ def validate_integer(option: str, value: Any) -> int:
         try:
             return int(value)
         except ValueError:
-            raise ValueError("The value of %s must be an integer" % (option,))
-    raise TypeError("Wrong type for %s, value must be an integer" % (option,))
+            raise ValueError(f"The value of {option} must be an integer")
+    raise TypeError(f"Wrong type for {option}, value must be an integer")
 
 
 def validate_positive_integer(option: str, value: Any) -> int:
     """Validate that 'value' is a positive integer, which does not include 0."""
     val = validate_integer(option, value)
     if val <= 0:
-        raise ValueError("The value of %s must be a positive integer" % (option,))
+        raise ValueError(f"The value of {option} must be a positive integer")
     return val
 
 
@@ -209,7 +211,7 @@ def validate_non_negative_integer(option: str, value: Any) -> int:
     """Validate that 'value' is a positive integer or 0."""
     val = validate_integer(option, value)
     if val < 0:
-        raise ValueError("The value of %s must be a non negative integer" % (option,))
+        raise ValueError(f"The value of {option} must be a non negative integer")
     return val
 
 
@@ -220,7 +222,7 @@ def validate_readable(option: str, value: Any) -> Optional[str]:
     # First make sure its a string py3.3 open(True, 'r') succeeds
     # Used in ssl cert checking due to poor ssl module error reporting
     value = validate_string(option, value)
-    open(value, "r").close()
+    open(value).close()
     return value
 
 
@@ -242,7 +244,7 @@ def validate_string(option: str, value: Any) -> str:
     """Validates that 'value' is an instance of `str`."""
     if isinstance(value, str):
         return value
-    raise TypeError("Wrong type for %s, value must be an instance of str" % (option,))
+    raise TypeError(f"Wrong type for {option}, value must be an instance of str")
 
 
 def validate_string_or_none(option: str, value: Any) -> Optional[str]:
@@ -261,7 +263,7 @@ def validate_int_or_basestring(option: str, value: Any) -> Union[int, str]:
             return int(value)
         except ValueError:
             return value
-    raise TypeError("Wrong type for %s, value must be an integer or a string" % (option,))
+    raise TypeError(f"Wrong type for {option}, value must be an integer or a string")
 
 
 def validate_non_negative_int_or_basestring(option: Any, value: Any) -> Union[int, str]:
@@ -274,16 +276,14 @@ def validate_non_negative_int_or_basestring(option: Any, value: Any) -> Union[in
         except ValueError:
             return value
         return validate_non_negative_integer(option, val)
-    raise TypeError(
-        "Wrong type for %s, value must be an non negative integer or a string" % (option,)
-    )
+    raise TypeError(f"Wrong type for {option}, value must be an non negative integer or a string")
 
 
 def validate_positive_float(option: str, value: Any) -> float:
     """Validates that 'value' is a float, or can be converted to one, and is
     positive.
     """
-    errmsg = "%s must be an integer or float" % (option,)
+    errmsg = f"{option} must be an integer or float"
     try:
         value = float(value)
     except ValueError:
@@ -294,7 +294,7 @@ def validate_positive_float(option: str, value: Any) -> float:
     # float('inf') doesn't work in 2.4 or 2.5 on Windows, so just cap floats at
     # one billion - this is a reasonable approximation for infinity
     if not 0 < value < 1e9:
-        raise ValueError("%s must be greater than 0 and less than one billion" % (option,))
+        raise ValueError(f"{option} must be greater than 0 and less than one billion")
     return value
 
 
@@ -323,7 +323,7 @@ def validate_timeout_or_zero(option: str, value: Any) -> float:
     config error.
     """
     if value is None:
-        raise ConfigurationError("%s cannot be None" % (option,))
+        raise ConfigurationError(f"{option} cannot be None")
     if value == 0 or value == "0":
         return 0
     return validate_positive_float(option, value) / 1000.0
@@ -359,7 +359,7 @@ def validate_max_staleness(option: str, value: Any) -> int:
 def validate_read_preference(dummy: Any, value: Any) -> _ServerMode:
     """Validate a read preference."""
     if not isinstance(value, _ServerMode):
-        raise TypeError("%r is not a read preference." % (value,))
+        raise TypeError(f"{value!r} is not a read preference.")
     return value
 
 
@@ -371,14 +371,14 @@ def validate_read_preference_mode(dummy: Any, value: Any) -> _ServerMode:
        mode.
     """
     if value not in _MONGOS_MODES:
-        raise ValueError("%s is not a valid read preference" % (value,))
+        raise ValueError(f"{value} is not a valid read preference")
     return value
 
 
 def validate_auth_mechanism(option: str, value: Any) -> str:
     """Validate the authMechanism URI option."""
     if value not in MECHANISMS:
-        raise ValueError("%s must be in %s" % (option, tuple(MECHANISMS)))
+        raise ValueError(f"{option} must be in {tuple(MECHANISMS)}")
     return value
 
 
@@ -388,9 +388,9 @@ def validate_uuid_representation(dummy: Any, value: Any) -> int:
         return _UUID_REPRESENTATIONS[value]
     except KeyError:
         raise ValueError(
-            "%s is an invalid UUID representation. "
+            "{} is an invalid UUID representation. "
             "Must be one of "
-            "%s" % (value, tuple(_UUID_REPRESENTATIONS))
+            "{}".format(value, tuple(_UUID_REPRESENTATIONS))
         )
 
 
@@ -411,19 +411,53 @@ def validate_read_preference_tags(name: str, value: Any) -> List[Dict[str, str]]
                 tags[unquote_plus(key)] = unquote_plus(val)
             tag_sets.append(tags)
         except Exception:
-            raise ValueError("%r not a valid value for %s" % (tag_set, name))
+            raise ValueError(f"{tag_set!r} not a valid value for {name}")
     return tag_sets
 
 
 _MECHANISM_PROPS = frozenset(
-    ["SERVICE_NAME", "CANONICALIZE_HOST_NAME", "SERVICE_REALM", "AWS_SESSION_TOKEN"]
+    [
+        "SERVICE_NAME",
+        "CANONICALIZE_HOST_NAME",
+        "SERVICE_REALM",
+        "AWS_SESSION_TOKEN",
+        "PROVIDER_NAME",
+    ]
 )
 
 
 def validate_auth_mechanism_properties(option: str, value: Any) -> Dict[str, Union[bool, str]]:
     """Validate authMechanismProperties."""
-    value = validate_string(option, value)
     props: Dict[str, Any] = {}
+    if not isinstance(value, str):
+        if not isinstance(value, dict):
+            raise ValueError("Auth mechanism properties must be given as a string or a dictionary")
+        for key, value in value.items():
+            if isinstance(value, str):
+                props[key] = value
+            elif isinstance(value, bool):
+                props[key] = str(value).lower()
+            elif key in ["allowed_hosts"] and isinstance(value, list):
+                props[key] = value
+            elif inspect.isfunction(value):
+                signature = inspect.signature(value)
+                if key == "request_token_callback":
+                    expected_params = 2
+                elif key == "refresh_token_callback":
+                    expected_params = 2
+                else:
+                    raise ValueError(f"Unrecognized Auth mechanism function {key}")
+                if len(signature.parameters) != expected_params:
+                    msg = f"{key} must accept {expected_params} parameters"
+                    raise ValueError(msg)
+                props[key] = value
+            else:
+                raise ValueError(
+                    "Auth mechanism property values must be strings or callback functions"
+                )
+        return props
+
+    value = validate_string(option, value)
     for opt in value.split(","):
         try:
             key, val = opt.split(":")
@@ -437,13 +471,13 @@ def validate_auth_mechanism_properties(option: str, value: Any) -> Dict[str, Uni
             raise ValueError(
                 "auth mechanism properties must be "
                 "key:value pairs like SERVICE_NAME:"
-                "mongodb, not %s." % (opt,)
+                "mongodb, not {}.".format(opt)
             )
         if key not in _MECHANISM_PROPS:
             raise ValueError(
-                "%s is not a supported auth "
+                "{} is not a supported auth "
                 "mechanism property. Must be one of "
-                "%s." % (key, tuple(_MECHANISM_PROPS))
+                "{}.".format(key, tuple(_MECHANISM_PROPS))
             )
         if key == "CANONICALIZE_HOST_NAME":
             props[key] = validate_boolean_or_string(key, val)
@@ -467,9 +501,9 @@ def validate_document_class(
             is_mapping = issubclass(value.__origin__, abc.MutableMapping)
     if not is_mapping and not issubclass(value, RawBSONDocument):
         raise TypeError(
-            "%s must be dict, bson.son.SON, "
+            "{} must be dict, bson.son.SON, "
             "bson.raw_bson.RawBSONDocument, or a "
-            "sublass of collections.MutableMapping" % (option,)
+            "subclass of collections.MutableMapping".format(option)
         )
     return value
 
@@ -477,14 +511,14 @@ def validate_document_class(
 def validate_type_registry(option: Any, value: Any) -> Optional[TypeRegistry]:
     """Validate the type_registry option."""
     if value is not None and not isinstance(value, TypeRegistry):
-        raise TypeError("%s must be an instance of %s" % (option, TypeRegistry))
+        raise TypeError(f"{option} must be an instance of {TypeRegistry}")
     return value
 
 
 def validate_list(option: str, value: Any) -> List:
     """Validates that 'value' is a list."""
     if not isinstance(value, list):
-        raise TypeError("%s must be a list" % (option,))
+        raise TypeError(f"{option} must be a list")
     return value
 
 
@@ -499,9 +533,9 @@ def validate_list_or_mapping(option: Any, value: Any) -> None:
     """Validates that 'value' is a list or a document."""
     if not isinstance(value, (abc.Mapping, list)):
         raise TypeError(
-            "%s must either be a list or an instance of dict, "
+            "{} must either be a list or an instance of dict, "
             "bson.son.SON, or any other type that inherits from "
-            "collections.Mapping" % (option,)
+            "collections.Mapping".format(option)
         )
 
 
@@ -509,9 +543,9 @@ def validate_is_mapping(option: str, value: Any) -> None:
     """Validate the type of method arguments that expect a document."""
     if not isinstance(value, abc.Mapping):
         raise TypeError(
-            "%s must be an instance of dict, bson.son.SON, or "
+            "{} must be an instance of dict, bson.son.SON, or "
             "any other type that inherits from "
-            "collections.Mapping" % (option,)
+            "collections.Mapping".format(option)
         )
 
 
@@ -519,10 +553,10 @@ def validate_is_document_type(option: str, value: Any) -> None:
     """Validate the type of method arguments that expect a MongoDB document."""
     if not isinstance(value, (abc.MutableMapping, RawBSONDocument)):
         raise TypeError(
-            "%s must be an instance of dict, bson.son.SON, "
+            "{} must be an instance of dict, bson.son.SON, "
             "bson.raw_bson.RawBSONDocument, or "
             "a type that inherits from "
-            "collections.MutableMapping" % (option,)
+            "collections.MutableMapping".format(option)
         )
 
 
@@ -533,7 +567,7 @@ def validate_appname_or_none(option: str, value: Any) -> Optional[str]:
     validate_string(option, value)
     # We need length in bytes, so encode utf8 first.
     if len(value.encode("utf-8")) > 128:
-        raise ValueError("%s must be <= 128 bytes" % (option,))
+        raise ValueError(f"{option} must be <= 128 bytes")
     return value
 
 
@@ -542,7 +576,7 @@ def validate_driver_or_none(option: Any, value: Any) -> Optional[DriverInfo]:
     if value is None:
         return value
     if not isinstance(value, DriverInfo):
-        raise TypeError("%s must be an instance of DriverInfo" % (option,))
+        raise TypeError(f"{option} must be an instance of DriverInfo")
     return value
 
 
@@ -551,7 +585,7 @@ def validate_server_api_or_none(option: Any, value: Any) -> Optional[ServerApi]:
     if value is None:
         return value
     if not isinstance(value, ServerApi):
-        raise TypeError("%s must be an instance of ServerApi" % (option,))
+        raise TypeError(f"{option} must be an instance of ServerApi")
     return value
 
 
@@ -560,7 +594,7 @@ def validate_is_callable_or_none(option: Any, value: Any) -> Optional[Callable]:
     if value is None:
         return value
     if not callable(value):
-        raise ValueError("%s must be a callable" % (option,))
+        raise ValueError(f"{option} must be a callable")
     return value
 
 
@@ -594,9 +628,9 @@ def validate_unicode_decode_error_handler(dummy: Any, value: str) -> str:
     """Validate the Unicode decode error handler option of CodecOptions."""
     if value not in _UNICODE_DECODE_ERROR_HANDLERS:
         raise ValueError(
-            "%s is an invalid Unicode decode error handler. "
+            "{} is an invalid Unicode decode error handler. "
             "Must be one of "
-            "%s" % (value, tuple(_UNICODE_DECODE_ERROR_HANDLERS))
+            "{}".format(value, tuple(_UNICODE_DECODE_ERROR_HANDLERS))
         )
     return value
 
@@ -615,11 +649,26 @@ def validate_auto_encryption_opts_or_none(option: Any, value: Any) -> Optional[A
     from pymongo.encryption_options import AutoEncryptionOpts
 
     if not isinstance(value, AutoEncryptionOpts):
-        raise TypeError("%s must be an instance of AutoEncryptionOpts" % (option,))
+        raise TypeError(f"{option} must be an instance of AutoEncryptionOpts")
 
     return value
 
 
+def validate_datetime_conversion(option: Any, value: Any) -> Optional[DatetimeConversion]:
+    """Validate a DatetimeConversion string."""
+    if value is None:
+        return DatetimeConversion.DATETIME
+
+    if isinstance(value, str):
+        if value.isdigit():
+            return DatetimeConversion(int(value))
+        return DatetimeConversion[value]
+    elif isinstance(value, int):
+        return DatetimeConversion(value)
+
+    raise TypeError(f"{option} must be a str or int representing DatetimeConversion")
+
+
 # Dictionary where keys are the names of public URI options, and values
 # are lists of aliases for that option.
 URI_OPTIONS_ALIAS_MAP: Dict[str, List[str]] = {
@@ -684,6 +733,7 @@ NONSPEC_OPTIONS_VALIDATOR_MAP: Dict[str, Callable[[Any, Any], Any]] = {
     "uuidrepresentation": validate_uuid_representation,
     "waitqueuemultiple": validate_non_negative_integer_or_none,
     "waitqueuetimeoutms": validate_timeout_or_none,
+    "datetime_conversion": validate_datetime_conversion,
 }
 
 # Dictionary where keys are the names of keyword-only options for the
@@ -699,6 +749,7 @@ KW_VALIDATORS: Dict[str, Callable[[Any, Any], Any]] = {
     "password": validate_string_or_none,
     "server_selector": validate_is_callable_or_none,
     "auto_encryption_opts": validate_auto_encryption_opts_or_none,
+    "authoidcallowedhosts": validate_list,
 }
 
 # Dictionary where keys are any URI option name, and values are the
@@ -753,7 +804,7 @@ def validate_auth_option(option: str, value: Any) -> Tuple[str, Any]:
     """Validate optional authentication parameters."""
     lower, value = validate(option, value)
     if lower not in _AUTH_OPTIONS:
-        raise ConfigurationError("Unknown authentication option: %s" % (option,))
+        raise ConfigurationError(f"Unknown authentication option: {option}")
     return option, value
 
 
@@ -780,12 +831,21 @@ def get_validated_options(
     validated_options: MutableMapping[str, Any]
     if isinstance(options, _CaseInsensitiveDictionary):
         validated_options = _CaseInsensitiveDictionary()
-        get_normed_key = lambda x: x  # noqa: E731
-        get_setter_key = lambda x: options.cased_key(x)  # noqa: E731
+
+        def get_normed_key(x: str) -> str:
+            return x  # noqa: E731
+
+        def get_setter_key(x: str) -> str:
+            return options.cased_key(x)  # type: ignore[attr-defined] # noqa: E731
+
     else:
         validated_options = {}
-        get_normed_key = lambda x: x.lower()  # noqa: E731
-        get_setter_key = lambda x: x  # noqa: E731
+
+        def get_normed_key(x: str) -> str:
+            return x.lower()  # noqa: E731
+
+        def get_setter_key(x: str) -> str:
+            return x  # noqa: E731
 
     for opt, value in options.items():
         normed_key = get_normed_key(opt)
@@ -802,15 +862,11 @@ def get_validated_options(
     return validated_options
 
 
-def _esc_coll_name(encrypted_fields, name):
+def _esc_coll_name(encrypted_fields: Mapping[str, Any], name: str) -> Any:
     return encrypted_fields.get("escCollection", f"enxcol_.{name}.esc")
 
 
-def _ecc_coll_name(encrypted_fields, name):
-    return encrypted_fields.get("eccCollection", f"enxcol_.{name}.ecc")
-
-
-def _ecoc_coll_name(encrypted_fields, name):
+def _ecoc_coll_name(encrypted_fields: Mapping[str, Any], name: str) -> Any:
     return encrypted_fields.get("ecocCollection", f"enxcol_.{name}.ecoc")
 
 
@@ -818,7 +874,7 @@ def _ecoc_coll_name(encrypted_fields, name):
 WRITE_CONCERN_OPTIONS = frozenset(["w", "wtimeout", "wtimeoutms", "fsync", "j", "journal"])
 
 
-class BaseObject(object):
+class BaseObject:
     """A base class that provides attributes and methods common
     to multiple pymongo classes.
 
@@ -838,9 +894,9 @@ class BaseObject(object):
 
         if not isinstance(read_preference, _ServerMode):
             raise TypeError(
-                "%r is not valid for read_preference. See "
+                "{!r} is not valid for read_preference. See "
                 "pymongo.read_preferences for valid "
-                "options." % (read_preference,)
+                "options.".format(read_preference)
             )
         self.__read_preference = read_preference
 
@@ -871,7 +927,7 @@ class BaseObject(object):
         """
         return self.__write_concern
 
-    def _write_concern_for(self, session):
+    def _write_concern_for(self, session: Optional[ClientSession]) -> WriteConcern:
         """Read only access to the write concern of this instance or session."""
         # Override this operation's write concern with the transaction's.
         if session and session.in_transaction:
@@ -887,7 +943,7 @@ class BaseObject(object):
         """
         return self.__read_preference
 
-    def _read_preference_for(self, session):
+    def _read_preference_for(self, session: Optional[ClientSession]) -> _ServerMode:
         """Read only access to the read preference of this instance or session."""
         # Override this operation's read preference with the transaction's.
         if session:
@@ -904,38 +960,38 @@ class BaseObject(object):
         return self.__read_concern
 
 
-class _CaseInsensitiveDictionary(abc.MutableMapping):
-    def __init__(self, *args, **kwargs):
-        self.__casedkeys = {}
-        self.__data = {}
+class _CaseInsensitiveDictionary(MutableMapping[str, Any]):
+    def __init__(self, *args: Any, **kwargs: Any):
+        self.__casedkeys: Dict[str, Any] = {}
+        self.__data: Dict[str, Any] = {}
         self.update(dict(*args, **kwargs))
 
-    def __contains__(self, key):
+    def __contains__(self, key: str) -> bool:  # type: ignore[override]
         return key.lower() in self.__data
 
-    def __len__(self):
+    def __len__(self) -> int:
         return len(self.__data)
 
-    def __iter__(self):
+    def __iter__(self) -> Iterator[str]:
         return (key for key in self.__casedkeys)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return str({self.__casedkeys[k]: self.__data[k] for k in self})
 
-    def __setitem__(self, key, value):
+    def __setitem__(self, key: str, value: Any) -> None:
         lc_key = key.lower()
         self.__casedkeys[lc_key] = key
         self.__data[lc_key] = value
 
-    def __getitem__(self, key):
+    def __getitem__(self, key: str) -> Any:
         return self.__data[key.lower()]
 
-    def __delitem__(self, key):
+    def __delitem__(self, key: str) -> None:
         lc_key = key.lower()
         del self.__casedkeys[lc_key]
         del self.__data[lc_key]
 
-    def __eq__(self, other):
+    def __eq__(self, other: Any) -> bool:
         if not isinstance(other, abc.Mapping):
             return NotImplemented
         if len(self) != len(other):
@@ -946,24 +1002,32 @@ class _CaseInsensitiveDictionary(abc.MutableMapping):
 
         return True
 
-    def get(self, key, default=None):
+    def get(self, key: str, default: Optional[Any] = None) -> Any:
         return self.__data.get(key.lower(), default)
 
-    def pop(self, key, *args, **kwargs):
+    def pop(self, key: str, *args: Any, **kwargs: Any) -> Any:
         lc_key = key.lower()
         self.__casedkeys.pop(lc_key, None)
         return self.__data.pop(lc_key, *args, **kwargs)
 
-    def popitem(self):
+    def popitem(self) -> Tuple[str, Any]:
         lc_key, cased_key = self.__casedkeys.popitem()
         value = self.__data.pop(lc_key)
         return cased_key, value
 
-    def clear(self):
+    def clear(self) -> None:
         self.__casedkeys.clear()
         self.__data.clear()
 
-    def setdefault(self, key, default=None):
+    @overload
+    def setdefault(self, key: str, default: None = None) -> Optional[Any]:
+        ...
+
+    @overload
+    def setdefault(self, key: str, default: Any) -> Any:
+        ...
+
+    def setdefault(self, key: str, default: Optional[Any] = None) -> Optional[Any]:
         lc_key = key.lower()
         if key in self:
             return self.__data[lc_key]
@@ -972,7 +1036,7 @@ class _CaseInsensitiveDictionary(abc.MutableMapping):
             self.__data[lc_key] = default
             return default
 
-    def update(self, other):
+    def update(self, other: Mapping[str, Any]) -> None:  # type: ignore[override]
         if isinstance(other, _CaseInsensitiveDictionary):
             for key in other:
                 self[other.cased_key(key)] = other[key]
@@ -980,5 +1044,5 @@ class _CaseInsensitiveDictionary(abc.MutableMapping):
             for key in other:
                 self[key] = other[key]
 
-    def cased_key(self, key):
+    def cased_key(self, key: str) -> Any:
         return self.__casedkeys[key.lower()]
