@@ -13,12 +13,25 @@
 # limitations under the License.
 
 """Support for explicit client-side field level encryption."""
+from __future__ import annotations
 
 import contextlib
 import enum
 import socket
 import weakref
-from typing import Any, Generic, Mapping, Optional, Sequence
+from copy import deepcopy
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Dict,
+    Generic,
+    Iterator,
+    Mapping,
+    MutableMapping,
+    Optional,
+    Sequence,
+    Tuple,
+)
 
 try:
     from pymongocrypt.auto_encrypter import AutoEncrypter
@@ -39,19 +52,24 @@ from bson.errors import BSONError
 from bson.raw_bson import DEFAULT_RAW_BSON_OPTIONS, RawBSONDocument, _inflate_bson
 from bson.son import SON
 from pymongo import _csot
+from pymongo.collection import Collection
+from pymongo.common import CONNECT_TIMEOUT
 from pymongo.cursor import Cursor
 from pymongo.daemon import _spawn_daemon
-from pymongo.encryption_options import AutoEncryptionOpts
+from pymongo.database import Database
+from pymongo.encryption_options import AutoEncryptionOpts, RangeOpts
 from pymongo.errors import (
     ConfigurationError,
+    EncryptedCollectionError,
     EncryptionError,
     InvalidOperation,
+    PyMongoError,
     ServerSelectionTimeoutError,
 )
 from pymongo.mongo_client import MongoClient
 from pymongo.network import BLOCKING_IO_ERRORS
 from pymongo.operations import UpdateOne
-from pymongo.pool import PoolOptions, _configured_socket
+from pymongo.pool import PoolOptions, _configured_socket, _raise_connection_failure
 from pymongo.read_concern import ReadConcern
 from pymongo.results import BulkWriteResult, DeleteResult
 from pymongo.ssl_support import get_ssl_context
@@ -59,8 +77,11 @@ from pymongo.typings import _DocumentType
 from pymongo.uri_parser import parse_host
 from pymongo.write_concern import WriteConcern
 
+if TYPE_CHECKING:
+    from pymongocrypt.mongocrypt import MongoCryptKmsContext
+
 _HTTPS_PORT = 443
-_KMS_CONNECT_TIMEOUT = 10  # TODO: CDRIVER-3262 will define this value.
+_KMS_CONNECT_TIMEOUT = CONNECT_TIMEOUT  # CDRIVER-3262 redefined this value to CONNECT_TIMEOUT
 _MONGOCRYPTD_TIMEOUT_MS = 10000
 
 
@@ -71,7 +92,7 @@ _KEY_VAULT_OPTS = CodecOptions(document_class=RawBSONDocument)
 
 
 @contextlib.contextmanager
-def _wrap_encryption_errors():
+def _wrap_encryption_errors() -> Iterator[None]:
     """Context manager to wrap encryption related errors."""
     try:
         yield
@@ -83,8 +104,14 @@ def _wrap_encryption_errors():
         raise EncryptionError(exc)
 
 
-class _EncryptionIO(MongoCryptCallback):  # type: ignore
-    def __init__(self, client, key_vault_coll, mongocryptd_client, opts):
+class _EncryptionIO(MongoCryptCallback):  # type: ignore[misc]
+    def __init__(
+        self,
+        client: Optional[MongoClient],
+        key_vault_coll: Collection,
+        mongocryptd_client: Optional[MongoClient],
+        opts: AutoEncryptionOpts,
+    ):
         """Internal class to perform I/O on behalf of pymongocrypt."""
         self.client_ref: Any
         # Use a weak ref to break reference cycle.
@@ -92,7 +119,7 @@ class _EncryptionIO(MongoCryptCallback):  # type: ignore
             self.client_ref = weakref.ref(client)
         else:
             self.client_ref = None
-        self.key_vault_coll = key_vault_coll.with_options(
+        self.key_vault_coll: Optional[Collection] = key_vault_coll.with_options(
             codec_options=_KEY_VAULT_OPTS,
             read_concern=ReadConcern(level="majority"),
             write_concern=WriteConcern(w="majority"),
@@ -101,7 +128,7 @@ class _EncryptionIO(MongoCryptCallback):  # type: ignore
         self.opts = opts
         self._spawned = False
 
-    def kms_request(self, kms_context):
+    def kms_request(self, kms_context: MongoCryptKmsContext) -> None:
         """Complete a KMS request.
 
         :Parameters:
@@ -134,22 +161,28 @@ class _EncryptionIO(MongoCryptCallback):  # type: ignore
             ssl_context=ctx,
         )
         host, port = parse_host(endpoint, _HTTPS_PORT)
-        conn = _configured_socket((host, port), opts)
         try:
-            conn.sendall(message)
-            while kms_context.bytes_needed > 0:
-                # CSOT: update timeout.
-                conn.settimeout(max(_csot.clamp_remaining(_KMS_CONNECT_TIMEOUT), 0))
-                data = conn.recv(kms_context.bytes_needed)
-                if not data:
-                    raise OSError("KMS connection closed")
-                kms_context.feed(data)
-        except BLOCKING_IO_ERRORS:
-            raise socket.timeout("timed out")
-        finally:
-            conn.close()
+            conn = _configured_socket((host, port), opts)
+            try:
+                conn.sendall(message)
+                while kms_context.bytes_needed > 0:
+                    # CSOT: update timeout.
+                    conn.settimeout(max(_csot.clamp_remaining(_KMS_CONNECT_TIMEOUT), 0))
+                    data = conn.recv(kms_context.bytes_needed)
+                    if not data:
+                        raise OSError("KMS connection closed")
+                    kms_context.feed(data)
+            except BLOCKING_IO_ERRORS:
+                raise socket.timeout("timed out")
+            finally:
+                conn.close()
+        except (PyMongoError, MongoCryptError):
+            raise  # Propagate pymongo errors directly.
+        except Exception as error:
+            # Wrap I/O errors in PyMongo exceptions.
+            _raise_connection_failure((host, port), error)
 
-    def collection_info(self, database, filter):
+    def collection_info(self, database: Database, filter: bytes) -> Optional[bytes]:
         """Get the collection info for a namespace.
 
         The returned collection info is passed to libmongocrypt which reads
@@ -165,8 +198,9 @@ class _EncryptionIO(MongoCryptCallback):  # type: ignore
         with self.client_ref()[database].list_collections(filter=RawBSONDocument(filter)) as cursor:
             for doc in cursor:
                 return _dict_to_bson(doc, False, _DATA_KEY_OPTS)
+            return None
 
-    def spawn(self):
+    def spawn(self) -> None:
         """Spawn mongocryptd.
 
         Note this method is thread safe; at most one mongocryptd will start
@@ -177,7 +211,7 @@ class _EncryptionIO(MongoCryptCallback):  # type: ignore
         args.extend(self.opts._mongocryptd_spawn_args)
         _spawn_daemon(args)
 
-    def mark_command(self, database, cmd):
+    def mark_command(self, database: str, cmd: bytes) -> bytes:
         """Mark a command for encryption.
 
         :Parameters:
@@ -192,6 +226,7 @@ class _EncryptionIO(MongoCryptCallback):  # type: ignore
         # Database.command only supports mutable mappings so we need to decode
         # the raw BSON command first.
         inflated_cmd = _inflate_bson(cmd, DEFAULT_RAW_BSON_OPTIONS)
+        assert self.mongocryptd_client is not None
         try:
             res = self.mongocryptd_client[database].command(
                 inflated_cmd, codec_options=DEFAULT_RAW_BSON_OPTIONS
@@ -205,7 +240,7 @@ class _EncryptionIO(MongoCryptCallback):  # type: ignore
             )
         return res.raw
 
-    def fetch_keys(self, filter):
+    def fetch_keys(self, filter: bytes) -> Iterator[bytes]:
         """Yields one or more keys from the key vault.
 
         :Parameters:
@@ -214,11 +249,12 @@ class _EncryptionIO(MongoCryptCallback):  # type: ignore
         :Returns:
           A generator which yields the requested keys from the key vault.
         """
+        assert self.key_vault_coll is not None
         with self.key_vault_coll.find(RawBSONDocument(filter)) as cursor:
             for key in cursor:
                 yield key.raw
 
-    def insert_data_key(self, data_key):
+    def insert_data_key(self, data_key: bytes) -> Binary:
         """Insert a data key into the key vault.
 
         :Parameters:
@@ -232,10 +268,11 @@ class _EncryptionIO(MongoCryptCallback):  # type: ignore
         if not isinstance(data_key_id, Binary) or data_key_id.subtype != UUID_SUBTYPE:
             raise TypeError("data_key _id must be Binary with a UUID subtype")
 
+        assert self.key_vault_coll is not None
         self.key_vault_coll.insert_one(raw_doc)
         return data_key_id
 
-    def bson_encode(self, doc):
+    def bson_encode(self, doc: MutableMapping[str, Any]) -> bytes:
         """Encode a document to BSON.
 
         A document can be any mapping type (like :class:`dict`).
@@ -248,7 +285,7 @@ class _EncryptionIO(MongoCryptCallback):  # type: ignore
         """
         return encode(doc)
 
-    def close(self):
+    def close(self) -> None:
         """Release resources.
 
         Note it is not safe to call this method from __del__ or any GC hooks.
@@ -260,7 +297,7 @@ class _EncryptionIO(MongoCryptCallback):  # type: ignore
             self.mongocryptd_client = None
 
 
-class RewrapManyDataKeyResult(object):
+class RewrapManyDataKeyResult:
     """Result object returned by a :meth:`~ClientEncryption.rewrap_many_data_key` operation.
 
     .. versionadded:: 4.2
@@ -280,13 +317,14 @@ class RewrapManyDataKeyResult(object):
         return self._bulk_write_result
 
 
-class _Encrypter(object):
+class _Encrypter:
     """Encrypts and decrypts MongoDB commands.
 
     This class is used to support automatic encryption and decryption of
-    MongoDB commands."""
+    MongoDB commands.
+    """
 
-    def __init__(self, client, opts):
+    def __init__(self, client: MongoClient, opts: AutoEncryptionOpts):
         """Create a _Encrypter for a client.
 
         :Parameters:
@@ -305,7 +343,7 @@ class _Encrypter(object):
         self._bypass_auto_encryption = opts._bypass_auto_encryption
         self._internal_client = None
 
-        def _get_internal_client(encrypter, mongo_client):
+        def _get_internal_client(encrypter: _Encrypter, mongo_client: MongoClient) -> MongoClient:
             if mongo_client.options.pool_options.max_pool_size is None:
                 # Unlimited pool size, use the same client.
                 return mongo_client
@@ -348,7 +386,9 @@ class _Encrypter(object):
         )
         self._closed = False
 
-    def encrypt(self, database, cmd, codec_options):
+    def encrypt(
+        self, database: str, cmd: Mapping[str, Any], codec_options: CodecOptions
+    ) -> Dict[Any, Any]:
         """Encrypt a MongoDB command.
 
         :Parameters:
@@ -367,7 +407,7 @@ class _Encrypter(object):
             encrypt_cmd = _inflate_bson(encrypted_cmd, DEFAULT_RAW_BSON_OPTIONS)
             return encrypt_cmd
 
-    def decrypt(self, response):
+    def decrypt(self, response: bytes) -> Optional[bytes]:
         """Decrypt a MongoDB command response.
 
         :Parameters:
@@ -380,11 +420,11 @@ class _Encrypter(object):
         with _wrap_encryption_errors():
             return self._auto_encrypter.decrypt(response)
 
-    def _check_closed(self):
+    def _check_closed(self) -> None:
         if self._closed:
             raise InvalidOperation("Cannot use MongoClient after close")
 
-    def close(self):
+    def close(self) -> None:
         """Cleanup resources."""
         self._closed = True
         self._auto_encrypter.close()
@@ -403,26 +443,25 @@ class Algorithm(str, enum.Enum):
     INDEXED = "Indexed"
     """Indexed.
 
-    .. note:: Support for Queryable Encryption is in beta.
-       Backwards-breaking changes may be made before the final release.
-
     .. versionadded:: 4.2
     """
     UNINDEXED = "Unindexed"
     """Unindexed.
 
-    .. note:: Support for Queryable Encryption is in beta.
+    .. versionadded:: 4.2
+    """
+    RANGEPREVIEW = "RangePreview"
+    """RangePreview.
+
+    .. note:: Support for Range queries is in beta.
        Backwards-breaking changes may be made before the final release.
 
-    .. versionadded:: 4.2
+    .. versionadded:: 4.4
     """
 
 
 class QueryType(str, enum.Enum):
-    """**(BETA)** An enum that defines the supported values for explicit encryption query_type.
-
-    .. note:: Support for Queryable Encryption is in beta.
-       Backwards-breaking changes may be made before the final release.
+    """An enum that defines the supported values for explicit encryption query_type.
 
     .. versionadded:: 4.2
     """
@@ -430,6 +469,13 @@ class QueryType(str, enum.Enum):
     EQUALITY = "equality"
     """Used to encrypt a value for an equality query."""
 
+    RANGEPREVIEW = "rangePreview"
+    """Used to encrypt a value for a range query.
+
+    .. note:: Support for Range queries is in beta.
+       Backwards-breaking changes may be made before the final release.
+"""
+
 
 class ClientEncryption(Generic[_DocumentType]):
     """Explicit client-side field level encryption."""
@@ -541,6 +587,90 @@ class ClientEncryption(Generic[_DocumentType]):
         # Use the same key vault collection as the callback.
         self._key_vault_coll = self._io_callbacks.key_vault_coll
 
+    def create_encrypted_collection(
+        self,
+        database: Database,
+        name: str,
+        encrypted_fields: Mapping[str, Any],
+        kms_provider: Optional[str] = None,
+        master_key: Optional[Mapping[str, Any]] = None,
+        **kwargs: Any,
+    ) -> Tuple[Collection[_DocumentType], Mapping[str, Any]]:
+        """Create a collection with encryptedFields.
+
+        .. warning::
+            This function does not update the encryptedFieldsMap in the client's
+            AutoEncryptionOpts, thus the user must create a new client after calling this function with
+            the encryptedFields returned.
+
+        Normally collection creation is automatic. This method should
+        only be used to specify options on
+        creation. :class:`~pymongo.errors.EncryptionError` will be
+        raised if the collection already exists.
+
+        :Parameters:
+          - `name`: the name of the collection to create
+          - `encrypted_fields` (dict): Document that describes the encrypted fields for
+            Queryable Encryption. For example::
+
+              {
+                "escCollection": "enxcol_.encryptedCollection.esc",
+                "ecocCollection": "enxcol_.encryptedCollection.ecoc",
+                "fields": [
+                    {
+                        "path": "firstName",
+                        "keyId": Binary.from_uuid(UUID('00000000-0000-0000-0000-000000000000')),
+                        "bsonType": "string",
+                        "queries": {"queryType": "equality"}
+                    },
+                    {
+                        "path": "ssn",
+                        "keyId": Binary.from_uuid(UUID('04104104-1041-0410-4104-104104104104')),
+                        "bsonType": "string"
+                    }
+                  ]
+              }
+
+            The "keyId" may be set to ``None`` to auto-generate the data keys.
+          - `kms_provider` (optional): the KMS provider to be used
+          - `master_key` (optional): Identifies a KMS-specific key used to encrypt the
+            new data key. If the kmsProvider is "local" the `master_key` is
+            not applicable and may be omitted.
+          - `**kwargs` (optional): additional keyword arguments are the same as "create_collection".
+
+        All optional `create collection command`_ parameters should be passed
+        as keyword arguments to this method.
+        See the documentation for :meth:`~pymongo.database.Database.create_collection` for all valid options.
+
+        :Raises:
+          - :class:`~pymongo.errors.EncryptedCollectionError`: When either data-key creation or creating the collection fails.
+
+        .. versionadded:: 4.4
+
+        .. _create collection command:
+            https://mongodb.com/docs/manual/reference/command/create
+
+        """
+        encrypted_fields = deepcopy(encrypted_fields)
+        for i, field in enumerate(encrypted_fields["fields"]):
+            if isinstance(field, dict) and field.get("keyId") is None:
+                try:
+                    encrypted_fields["fields"][i]["keyId"] = self.create_data_key(
+                        kms_provider=kms_provider,  # type:ignore[arg-type]
+                        master_key=master_key,
+                    )
+                except EncryptionError as exc:
+                    raise EncryptedCollectionError(exc, encrypted_fields) from exc
+        kwargs["encryptedFields"] = encrypted_fields
+        kwargs["check_exists"] = False
+        try:
+            return (
+                database.create_collection(name=name, **kwargs),
+                encrypted_fields,
+            )
+        except Exception as exc:
+            raise EncryptedCollectionError(exc, encrypted_fields) from exc
+
     def create_data_key(
         self,
         kms_provider: str,
@@ -603,9 +733,9 @@ class ClientEncryption(Generic[_DocumentType]):
             name instead of by ``key_id``. The following example shows creating
             and referring to a data key by alternate name::
 
-              client_encryption.create_data_key("local", keyAltNames=["name1"])
+              client_encryption.create_data_key("local", key_alt_names=["name1"])
               # reference the key with the alternate name
-              client_encryption.encrypt("457-55-5462", keyAltName="name1",
+              client_encryption.encrypt("457-55-5462", key_alt_name="name1",
                                         algorithm=Algorithm.AEAD_AES_256_CBC_HMAC_SHA_512_Random)
           - `key_material` (optional): Sets the custom key material to be used
             by the data key for encryption and decryption.
@@ -627,6 +757,46 @@ class ClientEncryption(Generic[_DocumentType]):
                 key_material=key_material,
             )
 
+    def _encrypt_helper(
+        self,
+        value: Any,
+        algorithm: str,
+        key_id: Optional[Binary] = None,
+        key_alt_name: Optional[str] = None,
+        query_type: Optional[str] = None,
+        contention_factor: Optional[int] = None,
+        range_opts: Optional[RangeOpts] = None,
+        is_expression: bool = False,
+    ) -> Any:
+        self._check_closed()
+        if key_id is not None and not (
+            isinstance(key_id, Binary) and key_id.subtype == UUID_SUBTYPE
+        ):
+            raise TypeError("key_id must be a bson.binary.Binary with subtype 4")
+
+        doc = encode(
+            {"v": value},
+            codec_options=self._codec_options,
+        )
+        range_opts_bytes = None
+        if range_opts:
+            range_opts_bytes = encode(
+                range_opts.document,
+                codec_options=self._codec_options,
+            )
+        with _wrap_encryption_errors():
+            encrypted_doc = self._encryption.encrypt(
+                value=doc,
+                algorithm=algorithm,
+                key_id=key_id,
+                key_alt_name=key_alt_name,
+                query_type=query_type,
+                contention_factor=contention_factor,
+                range_opts=range_opts_bytes,
+                is_expression=is_expression,
+            )
+            return decode(encrypted_doc)["v"]
+
     def encrypt(
         self,
         value: Any,
@@ -635,6 +805,7 @@ class ClientEncryption(Generic[_DocumentType]):
         key_alt_name: Optional[str] = None,
         query_type: Optional[str] = None,
         contention_factor: Optional[int] = None,
+        range_opts: Optional[RangeOpts] = None,
     ) -> Binary:
         """Encrypt a BSON value with a given key and algorithm.
 
@@ -649,41 +820,76 @@ class ClientEncryption(Generic[_DocumentType]):
             :class:`~bson.binary.Binary` with subtype 4 (
             :attr:`~bson.binary.UUID_SUBTYPE`).
           - `key_alt_name`: Identifies a key vault document by 'keyAltName'.
-          - `query_type` (str): **(BETA)** The query type to execute. See
-            :class:`QueryType` for valid options.
-          - `contention_factor` (int): **(BETA)** The contention factor to use
+          - `query_type` (str): The query type to execute. See :class:`QueryType` for valid options.
+          - `contention_factor` (int): The contention factor to use
             when the algorithm is :attr:`Algorithm.INDEXED`.  An integer value
             *must* be given when the :attr:`Algorithm.INDEXED` algorithm is
             used.
-
-        .. note:: `query_type` and `contention_factor` are part of the
-           Queryable Encryption beta. Backwards-breaking changes may be made before the
-           final release.
+          - `range_opts`: Experimental only, not intended for public use.
 
         :Returns:
           The encrypted value, a :class:`~bson.binary.Binary` with subtype 6.
 
         .. versionchanged:: 4.2
            Added the `query_type` and `contention_factor` parameters.
-
         """
-        self._check_closed()
-        if key_id is not None and not (
-            isinstance(key_id, Binary) and key_id.subtype == UUID_SUBTYPE
-        ):
-            raise TypeError("key_id must be a bson.binary.Binary with subtype 4")
+        return self._encrypt_helper(
+            value=value,
+            algorithm=algorithm,
+            key_id=key_id,
+            key_alt_name=key_alt_name,
+            query_type=query_type,
+            contention_factor=contention_factor,
+            range_opts=range_opts,
+            is_expression=False,
+        )
 
-        doc = encode({"v": value}, codec_options=self._codec_options)
-        with _wrap_encryption_errors():
-            encrypted_doc = self._encryption.encrypt(
-                doc,
-                algorithm,
-                key_id=key_id,
-                key_alt_name=key_alt_name,
-                query_type=query_type,
-                contention_factor=contention_factor,
-            )
-            return decode(encrypted_doc)["v"]  # type: ignore[index]
+    def encrypt_expression(
+        self,
+        expression: Mapping[str, Any],
+        algorithm: str,
+        key_id: Optional[Binary] = None,
+        key_alt_name: Optional[str] = None,
+        query_type: Optional[str] = None,
+        contention_factor: Optional[int] = None,
+        range_opts: Optional[RangeOpts] = None,
+    ) -> RawBSONDocument:
+        """Encrypt a BSON expression with a given key and algorithm.
+
+        Note that exactly one of ``key_id`` or  ``key_alt_name`` must be
+        provided.
+
+        :Parameters:
+          - `expression`: The BSON aggregate or match expression to encrypt.
+          - `algorithm` (string): The encryption algorithm to use. See
+            :class:`Algorithm` for some valid options.
+          - `key_id`: Identifies a data key by ``_id`` which must be a
+            :class:`~bson.binary.Binary` with subtype 4 (
+            :attr:`~bson.binary.UUID_SUBTYPE`).
+          - `key_alt_name`: Identifies a key vault document by 'keyAltName'.
+          - `query_type` (str): The query type to execute. See
+            :class:`QueryType` for valid options.
+          - `contention_factor` (int): The contention factor to use
+            when the algorithm is :attr:`Algorithm.INDEXED`.  An integer value
+            *must* be given when the :attr:`Algorithm.INDEXED` algorithm is
+            used.
+          - `range_opts`: Experimental only, not intended for public use.
+
+        :Returns:
+          The encrypted expression, a :class:`~bson.RawBSONDocument`.
+
+        .. versionadded:: 4.4
+        """
+        return self._encrypt_helper(
+            value=expression,
+            algorithm=algorithm,
+            key_id=key_id,
+            key_alt_name=key_alt_name,
+            query_type=query_type,
+            contention_factor=contention_factor,
+            range_opts=range_opts,
+            is_expression=True,
+        )
 
     def decrypt(self, value: Binary) -> Any:
         """Decrypt an encrypted value.
@@ -718,6 +924,7 @@ class ClientEncryption(Generic[_DocumentType]):
         .. versionadded:: 4.2
         """
         self._check_closed()
+        assert self._key_vault_coll is not None
         return self._key_vault_coll.find_one({"_id": id})
 
     def get_keys(self) -> Cursor[RawBSONDocument]:
@@ -730,6 +937,7 @@ class ClientEncryption(Generic[_DocumentType]):
         .. versionadded:: 4.2
         """
         self._check_closed()
+        assert self._key_vault_coll is not None
         return self._key_vault_coll.find({})
 
     def delete_key(self, id: Binary) -> DeleteResult:
@@ -746,6 +954,7 @@ class ClientEncryption(Generic[_DocumentType]):
         .. versionadded:: 4.2
         """
         self._check_closed()
+        assert self._key_vault_coll is not None
         return self._key_vault_coll.delete_one({"_id": id})
 
     def add_key_alt_name(self, id: Binary, key_alt_name: str) -> Any:
@@ -764,6 +973,7 @@ class ClientEncryption(Generic[_DocumentType]):
         """
         self._check_closed()
         update = {"$addToSet": {"keyAltNames": key_alt_name}}
+        assert self._key_vault_coll is not None
         return self._key_vault_coll.find_one_and_update({"_id": id}, update)
 
     def get_key_by_alt_name(self, key_alt_name: str) -> Optional[RawBSONDocument]:
@@ -778,6 +988,7 @@ class ClientEncryption(Generic[_DocumentType]):
         .. versionadded:: 4.2
         """
         self._check_closed()
+        assert self._key_vault_coll is not None
         return self._key_vault_coll.find_one({"keyAltNames": key_alt_name})
 
     def remove_key_alt_name(self, id: Binary, key_alt_name: str) -> Optional[RawBSONDocument]:
@@ -815,6 +1026,7 @@ class ClientEncryption(Generic[_DocumentType]):
                 }
             }
         ]
+        assert self._key_vault_coll is not None
         return self._key_vault_coll.find_one_and_update({"_id": id}, pipeline)
 
     def rewrap_many_data_key(
@@ -835,8 +1047,27 @@ class ClientEncryption(Generic[_DocumentType]):
         :Returns:
           A :class:`RewrapManyDataKeyResult`.
 
+        This method allows you to re-encrypt all of your data-keys with a new CMK, or master key.
+        Note that this does *not* require re-encrypting any of the data in your encrypted collections,
+        but rather refreshes the key that protects the keys that encrypt the data:
+
+        .. code-block:: python
+
+           client_encryption.rewrap_many_data_key(
+               filter={"keyAltNames": "optional filter for which keys you want to update"},
+               master_key={
+                   "provider": "azure",  # replace with your cloud provider
+                   "master_key": {
+                       # put the rest of your master_key options here
+                       "key": "<your new key>"
+                   },
+               },
+           )
+
         .. versionadded:: 4.2
         """
+        if master_key is not None and provider is None:
+            raise ConfigurationError("A provider must be given if a master_key is given")
         self._check_closed()
         with _wrap_encryption_errors():
             raw_result = self._encryption.rewrap_many_data_key(filter, provider, master_key)
@@ -854,6 +1085,7 @@ class ClientEncryption(Generic[_DocumentType]):
             replacements.append(op)
         if not replacements:
             return RewrapManyDataKeyResult()
+        assert self._key_vault_coll is not None
         result = self._key_vault_coll.bulk_write(replacements)
         return RewrapManyDataKeyResult(result)
 
@@ -863,7 +1095,7 @@ class ClientEncryption(Generic[_DocumentType]):
     def __exit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
         self.close()
 
-    def _check_closed(self):
+    def _check_closed(self) -> None:
         if self._encryption is None:
             raise InvalidOperation("Cannot use closed ClientEncryption")
 
