@@ -13,21 +13,43 @@
 # limitations under the License.
 
 """Operation class definitions."""
-from typing import Any, Dict, List, Mapping, Optional, Sequence, Tuple, Union
+from __future__ import annotations
 
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Dict,
+    Generic,
+    List,
+    Mapping,
+    Optional,
+    Sequence,
+    Tuple,
+    Union,
+)
+
+from bson.raw_bson import RawBSONDocument
 from pymongo import helpers
 from pymongo.collation import validate_collation_or_none
 from pymongo.common import validate_boolean, validate_is_mapping, validate_list
 from pymongo.helpers import _gen_index_name, _index_document, _index_list
-from pymongo.typings import _CollationIn, _DocumentIn, _Pipeline
+from pymongo.typings import _CollationIn, _DocumentType, _Pipeline
+
+if TYPE_CHECKING:
+    from bson.son import SON
+    from pymongo.bulk import _Bulk
+
+# Hint supports index name, "myIndex", or list of either strings or index pairs: [('x', 1), ('y', -1), 'z'']
+_IndexList = Sequence[Union[str, Tuple[str, Union[int, str, Mapping[str, Any]]]]]
+_IndexKeyHint = Union[str, _IndexList]
 
 
-class InsertOne(object):
+class InsertOne(Generic[_DocumentType]):
     """Represents an insert_one operation."""
 
     __slots__ = ("_doc",)
 
-    def __init__(self, document: _DocumentIn) -> None:
+    def __init__(self, document: _DocumentType) -> None:
         """Create an InsertOne instance.
 
         For use with :meth:`~pymongo.collection.Collection.bulk_write`.
@@ -38,12 +60,12 @@ class InsertOne(object):
         """
         self._doc = document
 
-    def _add_to_bulk(self, bulkobj):
+    def _add_to_bulk(self, bulkobj: _Bulk) -> None:
         """Add this operation to the _Bulk instance `bulkobj`."""
-        bulkobj.add_insert(self._doc)
+        bulkobj.add_insert(self._doc)  # type: ignore[arg-type]
 
-    def __repr__(self):
-        return "InsertOne(%r)" % (self._doc,)
+    def __repr__(self) -> str:
+        return f"InsertOne({self._doc!r})"
 
     def __eq__(self, other: Any) -> bool:
         if type(other) == type(self):
@@ -54,11 +76,7 @@ class InsertOne(object):
         return not self == other
 
 
-_IndexList = Sequence[Tuple[str, Union[int, str, Mapping[str, Any]]]]
-_IndexKeyHint = Union[str, _IndexList]
-
-
-class DeleteOne(object):
+class DeleteOne:
     """Represents a delete_one operation."""
 
     __slots__ = ("_filter", "_collation", "_hint")
@@ -91,19 +109,24 @@ class DeleteOne(object):
         """
         if filter is not None:
             validate_is_mapping("filter", filter)
-        if hint is not None:
-            if not isinstance(hint, str):
-                hint = helpers._index_document(hint)
+        if hint is not None and not isinstance(hint, str):
+            self._hint: Union[str, SON[str, Any], None] = helpers._index_document(hint)
+        else:
+            self._hint = hint
         self._filter = filter
         self._collation = collation
-        self._hint = hint
 
-    def _add_to_bulk(self, bulkobj):
+    def _add_to_bulk(self, bulkobj: _Bulk) -> None:
         """Add this operation to the _Bulk instance `bulkobj`."""
-        bulkobj.add_delete(self._filter, 1, collation=self._collation, hint=self._hint)
+        bulkobj.add_delete(
+            self._filter,
+            1,
+            collation=validate_collation_or_none(self._collation),
+            hint=self._hint,
+        )
 
-    def __repr__(self):
-        return "DeleteOne(%r, %r)" % (self._filter, self._collation)
+    def __repr__(self) -> str:
+        return f"DeleteOne({self._filter!r}, {self._collation!r})"
 
     def __eq__(self, other: Any) -> bool:
         if type(other) == type(self):
@@ -114,7 +137,7 @@ class DeleteOne(object):
         return not self == other
 
 
-class DeleteMany(object):
+class DeleteMany:
     """Represents a delete_many operation."""
 
     __slots__ = ("_filter", "_collation", "_hint")
@@ -147,19 +170,24 @@ class DeleteMany(object):
         """
         if filter is not None:
             validate_is_mapping("filter", filter)
-        if hint is not None:
-            if not isinstance(hint, str):
-                hint = helpers._index_document(hint)
+        if hint is not None and not isinstance(hint, str):
+            self._hint: Union[str, SON[str, Any], None] = helpers._index_document(hint)
+        else:
+            self._hint = hint
         self._filter = filter
         self._collation = collation
-        self._hint = hint
 
-    def _add_to_bulk(self, bulkobj):
+    def _add_to_bulk(self, bulkobj: _Bulk) -> None:
         """Add this operation to the _Bulk instance `bulkobj`."""
-        bulkobj.add_delete(self._filter, 0, collation=self._collation, hint=self._hint)
+        bulkobj.add_delete(
+            self._filter,
+            0,
+            collation=validate_collation_or_none(self._collation),
+            hint=self._hint,
+        )
 
-    def __repr__(self):
-        return "DeleteMany(%r, %r)" % (self._filter, self._collation)
+    def __repr__(self) -> str:
+        return f"DeleteMany({self._filter!r}, {self._collation!r})"
 
     def __eq__(self, other: Any) -> bool:
         if type(other) == type(self):
@@ -170,7 +198,7 @@ class DeleteMany(object):
         return not self == other
 
 
-class ReplaceOne(object):
+class ReplaceOne(Generic[_DocumentType]):
     """Represents a replace_one operation."""
 
     __slots__ = ("_filter", "_doc", "_upsert", "_collation", "_hint")
@@ -178,7 +206,7 @@ class ReplaceOne(object):
     def __init__(
         self,
         filter: Mapping[str, Any],
-        replacement: Mapping[str, Any],
+        replacement: Union[_DocumentType, RawBSONDocument],
         upsert: bool = False,
         collation: Optional[_CollationIn] = None,
         hint: Optional[_IndexKeyHint] = None,
@@ -210,25 +238,28 @@ class ReplaceOne(object):
             validate_is_mapping("filter", filter)
         if upsert is not None:
             validate_boolean("upsert", upsert)
-        if hint is not None:
-            if not isinstance(hint, str):
-                hint = helpers._index_document(hint)
-
+        if hint is not None and not isinstance(hint, str):
+            self._hint: Union[str, SON[str, Any], None] = helpers._index_document(hint)
+        else:
+            self._hint = hint
         self._filter = filter
         self._doc = replacement
         self._upsert = upsert
         self._collation = collation
-        self._hint = hint
 
-    def _add_to_bulk(self, bulkobj):
+    def _add_to_bulk(self, bulkobj: _Bulk) -> None:
         """Add this operation to the _Bulk instance `bulkobj`."""
         bulkobj.add_replace(
-            self._filter, self._doc, self._upsert, collation=self._collation, hint=self._hint
+            self._filter,
+            self._doc,
+            self._upsert,
+            collation=validate_collation_or_none(self._collation),
+            hint=self._hint,
         )
 
     def __eq__(self, other: Any) -> bool:
         if type(other) == type(self):
-            return (other._filter, other._doc, other._upsert, other._collation, other._hint) == (
+            return (other._filter, other._doc, other._upsert, other._collation, other._hint,) == (
                 self._filter,
                 self._doc,
                 self._upsert,
@@ -240,8 +271,8 @@ class ReplaceOne(object):
     def __ne__(self, other: Any) -> bool:
         return not self == other
 
-    def __repr__(self):
-        return "%s(%r, %r, %r, %r, %r)" % (
+    def __repr__(self) -> str:
+        return "{}({!r}, {!r}, {!r}, {!r}, {!r})".format(
             self.__class__.__name__,
             self._filter,
             self._doc,
@@ -251,31 +282,39 @@ class ReplaceOne(object):
         )
 
 
-class _UpdateOp(object):
+class _UpdateOp:
     """Private base class for update operations."""
 
     __slots__ = ("_filter", "_doc", "_upsert", "_collation", "_array_filters", "_hint")
 
-    def __init__(self, filter, doc, upsert, collation, array_filters, hint):
+    def __init__(
+        self,
+        filter: Mapping[str, Any],
+        doc: Union[Mapping[str, Any], _Pipeline],
+        upsert: bool,
+        collation: Optional[_CollationIn],
+        array_filters: Optional[List[Mapping[str, Any]]],
+        hint: Optional[_IndexKeyHint],
+    ):
         if filter is not None:
             validate_is_mapping("filter", filter)
         if upsert is not None:
             validate_boolean("upsert", upsert)
         if array_filters is not None:
             validate_list("array_filters", array_filters)
-        if hint is not None:
-            if not isinstance(hint, str):
-                hint = helpers._index_document(hint)
+        if hint is not None and not isinstance(hint, str):
+            self._hint: Union[str, SON[str, Any], None] = helpers._index_document(hint)
+        else:
+            self._hint = hint
 
         self._filter = filter
         self._doc = doc
         self._upsert = upsert
         self._collation = collation
         self._array_filters = array_filters
-        self._hint = hint
 
-    def __eq__(self, other):
-        if type(other) == type(self):
+    def __eq__(self, other: object) -> bool:
+        if isinstance(other, type(self)):
             return (
                 other._filter,
                 other._doc,
@@ -293,11 +332,8 @@ class _UpdateOp(object):
             )
         return NotImplemented
 
-    def __ne__(self, other):
-        return not self == other
-
-    def __repr__(self):
-        return "%s(%r, %r, %r, %r, %r, %r)" % (
+    def __repr__(self) -> str:
+        return "{}({!r}, {!r}, {!r}, {!r}, {!r}, {!r})".format(
             self.__class__.__name__,
             self._filter,
             self._doc,
@@ -351,16 +387,16 @@ class UpdateOne(_UpdateOp):
         .. versionchanged:: 3.5
            Added the `collation` option.
         """
-        super(UpdateOne, self).__init__(filter, update, upsert, collation, array_filters, hint)
+        super().__init__(filter, update, upsert, collation, array_filters, hint)
 
-    def _add_to_bulk(self, bulkobj):
+    def _add_to_bulk(self, bulkobj: _Bulk) -> None:
         """Add this operation to the _Bulk instance `bulkobj`."""
         bulkobj.add_update(
             self._filter,
             self._doc,
             False,
             self._upsert,
-            collation=self._collation,
+            collation=validate_collation_or_none(self._collation),
             array_filters=self._array_filters,
             hint=self._hint,
         )
@@ -409,22 +445,22 @@ class UpdateMany(_UpdateOp):
         .. versionchanged:: 3.5
            Added the `collation` option.
         """
-        super(UpdateMany, self).__init__(filter, update, upsert, collation, array_filters, hint)
+        super().__init__(filter, update, upsert, collation, array_filters, hint)
 
-    def _add_to_bulk(self, bulkobj):
+    def _add_to_bulk(self, bulkobj: _Bulk) -> None:
         """Add this operation to the _Bulk instance `bulkobj`."""
         bulkobj.add_update(
             self._filter,
             self._doc,
             True,
             self._upsert,
-            collation=self._collation,
+            collation=validate_collation_or_none(self._collation),
             array_filters=self._array_filters,
             hint=self._hint,
         )
 
 
-class IndexModel(object):
+class IndexModel:
     """Represents an index to create."""
 
     __slots__ = ("__document",)
@@ -434,10 +470,11 @@ class IndexModel(object):
 
         For use with :meth:`~pymongo.collection.Collection.create_indexes`.
 
-        Takes either a single key or a list of (key, direction) pairs.
-        The key(s) must be an instance of :class:`basestring`
-        (:class:`str` in python 3), and the direction(s) must be one of
-        (:data:`~pymongo.ASCENDING`, :data:`~pymongo.DESCENDING`,
+        Takes either a single key or a list containing (key, direction) pairs
+        or keys.  If no direction is given, :data:`~pymongo.ASCENDING` will
+        be assumed.
+        The key(s) must be an instance of :class:`str`, and the direction(s) must
+        be one of (:data:`~pymongo.ASCENDING`, :data:`~pymongo.DESCENDING`,
         :data:`~pymongo.GEO2D`, :data:`~pymongo.GEOSPHERE`,
         :data:`~pymongo.HASHED`, :data:`~pymongo.TEXT`).
 
@@ -476,11 +513,11 @@ class IndexModel(object):
         server version.
 
         :Parameters:
-          - `keys`: a single key or a list of (key, direction)
-            pairs specifying the index to create
+          - `keys`: a single key or a list containing (key, direction) pairs
+             or keys specifying the index to create.
           - `**kwargs` (optional): any additional index creation
             options (see the above list) should be passed as keyword
-            arguments
+            arguments.
 
         .. versionchanged:: 3.11
            Added the ``hidden`` option.
@@ -491,7 +528,7 @@ class IndexModel(object):
         .. _wildcard index: https://mongodb.com/docs/master/core/index-wildcard/
         """
         keys = _index_list(keys)
-        if "name" not in kwargs:
+        if kwargs.get("name") is None:
             kwargs["name"] = _gen_index_name(keys)
         kwargs["key"] = _index_document(keys)
         collation = validate_collation_or_none(kwargs.pop("collation", None))
@@ -505,3 +542,32 @@ class IndexModel(object):
         command.
         """
         return self.__document
+
+
+class SearchIndexModel:
+    """Represents a search index to create."""
+
+    __slots__ = "__document"
+
+    def __init__(self, definition: Mapping[str, Any], name: Optional[str] = None) -> None:
+        """Create a Search Index instance.
+
+        For use with :meth:`~pymongo.collection.Collection.create_search_index` and :meth:`~pymongo.collection.Collection.create_search_indexes`.
+
+        :Parameters:
+          - `definition` - The definition for this index.
+          - `name` (optional) - The name for this index, if present.
+
+        .. versionadded:: 4.5
+
+        .. note:: Search indexes require a MongoDB server version 7.0+ Atlas cluster.
+        """
+        if name is not None:
+            self.__document = dict(name=name, definition=definition)
+        else:
+            self.__document = dict(definition=definition)
+
+    @property
+    def document(self) -> Mapping[str, Any]:
+        """The document for this index."""
+        return self.__document
