@@ -15,8 +15,20 @@
 """Represent a deployment of MongoDB servers."""
 
 from random import sample
-from typing import Any, Callable, Dict, List, NamedTuple, Optional, Tuple
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    List,
+    Mapping,
+    MutableMapping,
+    NamedTuple,
+    Optional,
+    Tuple,
+    cast,
+)
 
+from bson.min_key import MinKey
 from bson.objectid import ObjectId
 from pymongo import common
 from pymongo.errors import ConfigurationError
@@ -46,7 +58,7 @@ SRV_POLLING_TOPOLOGIES: Tuple[int, int] = (TOPOLOGY_TYPE.Unknown, TOPOLOGY_TYPE.
 _ServerSelector = Callable[[List[ServerDescription]], List[ServerDescription]]
 
 
-class TopologyDescription(object):
+class TopologyDescription:
     def __init__(
         self,
         topology_type: int,
@@ -98,7 +110,7 @@ class TopologyDescription(object):
                 s.logical_session_timeout_minutes for s in readable_servers
             )
 
-    def _init_incompatible_err(self):
+    def _init_incompatible_err(self) -> None:
         """Internal compatibility check for non-load balanced topologies."""
         for s in self._server_descriptions.values():
             if not s.is_server_type_known:
@@ -170,7 +182,7 @@ class TopologyDescription(object):
             topology_type = self._topology_type
 
         # The default ServerDescription's type is Unknown.
-        sds = dict((address, ServerDescription(address)) for address in self._server_descriptions)
+        sds = {address: ServerDescription(address) for address in self._server_descriptions}
 
         return TopologyDescription(
             topology_type,
@@ -183,7 +195,8 @@ class TopologyDescription(object):
 
     def server_descriptions(self) -> Dict[_Address, ServerDescription]:
         """Dict of (address,
-        :class:`~pymongo.server_description.ServerDescription`)."""
+        :class:`~pymongo.server_description.ServerDescription`).
+        """
         return self._server_descriptions.copy()
 
     @property
@@ -251,14 +264,16 @@ class TopologyDescription(object):
     def srv_max_hosts(self) -> int:
         return self._topology_settings._srv_max_hosts
 
-    def _apply_local_threshold(self, selection):
+    def _apply_local_threshold(self, selection: Optional[Selection]) -> List[ServerDescription]:
         if not selection:
             return []
         # Round trip time in seconds.
-        fastest = min(s.round_trip_time for s in selection.server_descriptions)
+        fastest = min(cast(float, s.round_trip_time) for s in selection.server_descriptions)
         threshold = self._topology_settings.local_threshold_ms / 1000.0
         return [
-            s for s in selection.server_descriptions if (s.round_trip_time - fastest) <= threshold
+            s
+            for s in selection.server_descriptions
+            if (cast(float, s.round_trip_time) - fastest) <= threshold
         ]
 
     def apply_selector(
@@ -342,10 +357,10 @@ class TopologyDescription(object):
         """
         return self.has_readable_server(ReadPreference.PRIMARY)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         # Sort the servers by address.
         servers = sorted(self._server_descriptions.values(), key=lambda sd: sd.address)
-        return "<%s id: %s, topology_type: %s, servers: %r>" % (
+        return "<{} id: {}, topology_type: {}, servers: {!r}>".format(
             self.__class__.__name__,
             self._topology_settings._topology_id,
             self.topology_type_name,
@@ -399,8 +414,9 @@ def updated_topology_description(
         if set_name is not None and set_name != server_description.replica_set_name:
             error = ConfigurationError(
                 "client is configured to connect to a replica set named "
-                "'%s' but this node belongs to a set named '%s'"
-                % (set_name, server_description.replica_set_name)
+                "'{}' but this node belongs to a set named '{}'".format(
+                    set_name, server_description.replica_set_name
+                )
             )
             sds[address] = server_description.to_unknown(error=error)
         # Single type never changes.
@@ -469,7 +485,9 @@ def updated_topology_description(
     )
 
 
-def _updated_topology_description_srv_polling(topology_description, seedlist):
+def _updated_topology_description_srv_polling(
+    topology_description: TopologyDescription, seedlist: List[Tuple[str, Any]]
+) -> TopologyDescription:
     """Return an updated copy of a TopologyDescription.
 
     :Parameters:
@@ -494,7 +512,7 @@ def _updated_topology_description_srv_polling(topology_description, seedlist):
         new_hosts = set(seedlist) - set(sds.keys())
         n_to_add = topology_description.srv_max_hosts - len(sds)
         if n_to_add > 0:
-            seedlist = sample(new_hosts, min(n_to_add, len(new_hosts)))
+            seedlist = sample(sorted(new_hosts), min(n_to_add, len(new_hosts)))
         else:
             seedlist = []
     # Add SDs corresponding to servers recently added to the SRV record.
@@ -512,8 +530,12 @@ def _updated_topology_description_srv_polling(topology_description, seedlist):
 
 
 def _update_rs_from_primary(
-    sds, replica_set_name, server_description, max_set_version, max_election_id
-):
+    sds: MutableMapping[_Address, ServerDescription],
+    replica_set_name: Optional[str],
+    server_description: ServerDescription,
+    max_set_version: Optional[int],
+    max_election_id: Optional[ObjectId],
+) -> Tuple[int, Optional[str], Optional[int], Optional[ObjectId]]:
     """Update topology description from a primary's hello response.
 
     Pass in a dict of ServerDescriptions, current replica set name, the
@@ -530,26 +552,34 @@ def _update_rs_from_primary(
         # We found a primary but it doesn't have the replica_set_name
         # provided by the user.
         sds.pop(server_description.address)
-        return (_check_has_primary(sds), replica_set_name, max_set_version, max_election_id)
+        return _check_has_primary(sds), replica_set_name, max_set_version, max_election_id
 
-    max_election_tuple = max_set_version, max_election_id
-    if None not in server_description.election_tuple:
-        if (
-            None not in max_election_tuple
-            and max_election_tuple > server_description.election_tuple
+    if server_description.max_wire_version is None or server_description.max_wire_version < 17:
+        new_election_tuple: Tuple = (server_description.set_version, server_description.election_id)
+        max_election_tuple: Tuple = (max_set_version, max_election_id)
+        if None not in new_election_tuple:
+            if None not in max_election_tuple and new_election_tuple < max_election_tuple:
+                # Stale primary, set to type Unknown.
+                sds[server_description.address] = server_description.to_unknown()
+                return _check_has_primary(sds), replica_set_name, max_set_version, max_election_id
+            max_election_id = server_description.election_id
+
+        if server_description.set_version is not None and (
+            max_set_version is None or server_description.set_version > max_set_version
         ):
-
+            max_set_version = server_description.set_version
+    else:
+        new_election_tuple = server_description.election_id, server_description.set_version
+        max_election_tuple = max_election_id, max_set_version
+        new_election_safe = tuple(MinKey() if i is None else i for i in new_election_tuple)
+        max_election_safe = tuple(MinKey() if i is None else i for i in max_election_tuple)
+        if new_election_safe < max_election_safe:
             # Stale primary, set to type Unknown.
             sds[server_description.address] = server_description.to_unknown()
-            return (_check_has_primary(sds), replica_set_name, max_set_version, max_election_id)
-
-        max_election_id = server_description.election_id
-
-    if server_description.set_version is not None and (
-        max_set_version is None or server_description.set_version > max_set_version
-    ):
-
-        max_set_version = server_description.set_version
+            return _check_has_primary(sds), replica_set_name, max_set_version, max_election_id
+        else:
+            max_election_id = server_description.election_id
+            max_set_version = server_description.set_version
 
     # We've heard from the primary. Is it the same primary as before?
     for server in sds.values():
@@ -578,7 +608,11 @@ def _update_rs_from_primary(
     return (_check_has_primary(sds), replica_set_name, max_set_version, max_election_id)
 
 
-def _update_rs_with_primary_from_member(sds, replica_set_name, server_description):
+def _update_rs_with_primary_from_member(
+    sds: MutableMapping[_Address, ServerDescription],
+    replica_set_name: Optional[str],
+    server_description: ServerDescription,
+) -> int:
     """RS with known primary. Process a response from a non-primary.
 
     Pass in a dict of ServerDescriptions, current replica set name, and the
@@ -597,7 +631,11 @@ def _update_rs_with_primary_from_member(sds, replica_set_name, server_descriptio
     return _check_has_primary(sds)
 
 
-def _update_rs_no_primary_from_member(sds, replica_set_name, server_description):
+def _update_rs_no_primary_from_member(
+    sds: MutableMapping[_Address, ServerDescription],
+    replica_set_name: Optional[str],
+    server_description: ServerDescription,
+) -> Tuple[int, Optional[str]]:
     """RS without known primary. Update from a non-primary's response.
 
     Pass in a dict of ServerDescriptions, current replica set name, and the
@@ -625,7 +663,7 @@ def _update_rs_no_primary_from_member(sds, replica_set_name, server_description)
     return topology_type, replica_set_name
 
 
-def _check_has_primary(sds):
+def _check_has_primary(sds: Mapping[_Address, ServerDescription]) -> int:
     """Current topology type is ReplicaSetWithPrimary. Is primary still known?
 
     Pass in a dict of ServerDescriptions.
