@@ -17,7 +17,6 @@ from typing import (
     TYPE_CHECKING,
     Any,
     Mapping,
-    MutableMapping,
     Optional,
     Sequence,
     Tuple,
@@ -25,15 +24,35 @@ from typing import (
     Union,
 )
 
+from bson.typings import _DocumentOut, _DocumentType, _DocumentTypeArg
+
 if TYPE_CHECKING:
-    from bson.raw_bson import RawBSONDocument
     from pymongo.collation import Collation
 
 
 # Common Shared Types.
 _Address = Tuple[str, Optional[int]]
 _CollationIn = Union[Mapping[str, Any], "Collation"]
-_DocumentIn = Union[MutableMapping[str, Any], "RawBSONDocument"]
 _Pipeline = Sequence[Mapping[str, Any]]
-_DocumentOut = _DocumentIn
-_DocumentType = TypeVar("_DocumentType", bound=Mapping[str, Any])
+ClusterTime = Mapping[str, Any]
+
+_T = TypeVar("_T")
+
+
+def strip_optional(elem: Optional[_T]) -> _T:
+    """This function is to allow us to cast all of the elements of an iterator from Optional[_T] to _T
+    while inside a list comprehension.
+    """
+    assert elem is not None
+    return elem
+
+
+__all__ = [
+    "_DocumentOut",
+    "_DocumentType",
+    "_DocumentTypeArg",
+    "_Address",
+    "_CollationIn",
+    "_Pipeline",
+    "strip_optional",
+]
