@@ -19,7 +19,15 @@ from typing import Any, Dict, Optional, Union
 from pymongo.errors import ConfigurationError
 
 
-class WriteConcern(object):
+# Moved here to avoid a circular import.
+def validate_boolean(option: str, value: Any) -> bool:
+    """Validates that 'value' is True or False."""
+    if isinstance(value, bool):
+        return value
+    raise TypeError(f"{option} must be True or False, was: {option}={value}")
+
+
+class WriteConcern:
     """WriteConcern
 
     :Parameters:
@@ -65,13 +73,11 @@ class WriteConcern(object):
             self.__document["wtimeout"] = wtimeout
 
         if j is not None:
-            if not isinstance(j, bool):
-                raise TypeError("j must be True or False")
+            validate_boolean("j", j)
             self.__document["j"] = j
 
         if fsync is not None:
-            if not isinstance(fsync, bool):
-                raise TypeError("fsync must be True or False")
+            validate_boolean("fsync", fsync)
             if j and fsync:
                 raise ConfigurationError("Can't set both j and fsync at the same time")
             self.__document["fsync"] = fsync
@@ -112,8 +118,10 @@ class WriteConcern(object):
         """
         return self.__acknowledged
 
-    def __repr__(self):
-        return "WriteConcern(%s)" % (", ".join("%s=%s" % kvt for kvt in self.__document.items()),)
+    def __repr__(self) -> str:
+        return "WriteConcern({})".format(
+            ", ".join("{}={}".format(*kvt) for kvt in self.__document.items())
+        )
 
     def __eq__(self, other: Any) -> bool:
         if isinstance(other, WriteConcern):
